/*
 * ImageViewer.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.app;

import java.io.*;
import java.net.*;
import java.awt.*;
import misao.util.*;
import misao.pixy.*;

class ImageViewer extends Frame implements Runnable {
    ImageFile image_file;

    Thread thread = null;

    ImageCanvas image_canvas;
    TextField text_min;
    TextField text_max;
    Button btnView;

    MinMaxAverage min_max_average;

    public static void main ( String[] args ) throws Exception {
	if (args.length == 0) {
	    args = new String[1];

	    try {
		DataInputStream din = new DataInputStream(System.in);

		System.out.print("URL or filename of image: ");
		args[0] = din.readLine();
	    } catch ( IOException e ) {
		System.err.println("Cannot open System.in.");
		throw e;
	    }
	}

	URL url;
	try {
	    url = new URL(new URLString(args[0]).toString());
	} catch ( MalformedURLException e ) {
	    System.err.println("Cannot access to: " + args[0]);
	    throw e;
	}

	ImageViewer image_viewer = new ImageViewer(url);
    }

    public ImageViewer ( URL url ) throws Exception {
	thread = new Thread(this);
	thread.start();

	try {
	    image_file = new ImageFile(url);
	} catch ( Exception e ) {
	    throw e;
	}

	min_max_average = image_file.calculateMinMaxAverage();

	image_canvas = new ImageCanvas(image_file.getImage(), image_file.getSize());
	text_min = new TextField(String.valueOf(min_max_average.getMin()), 10);
	text_max = new TextField(String.valueOf(min_max_average.getMax()), 10);
	btnView = new Button("View");

	Panel panel = new Panel(new FlowLayout());
	panel.add(new Label("Min."));
	panel.add(text_min);
	panel.add(new Label("Max."));
	panel.add(text_max);
	panel.add(btnView);

	setLayout(new FlowLayout());
	add(panel);
	add(image_canvas);
	resize(image_file.getWidth(), image_file.getHeight() + 45 + 25);
	show();
    }

    public boolean action(Event e, Object o) {
	if (e.target instanceof Button) {
	    if (btnView.getLabel().equals(o)) {
		try {
		    double min = (double)(Float.valueOf(text_min.getText().trim()).floatValue());
		    double max = (double)(Float.valueOf(text_max.getText().trim()).floatValue());

		    image_canvas.setImage(image_file.getImage(min, max));
		    image_canvas.repaint();

		    return true;
		} catch ( NumberFormatException except ) {
		    System.err.println("Number format exception: " + text_min.getText() + " " + text_max.getText());
		} catch ( Exception except ) {
		}
	    }
	}
	return false;
    }

    public void run ( ) {
	while (true) {
	    try {
		Thread.sleep(500);
	    } catch (InterruptedException e) {
		System.err.println("sleep() is interrupted. " + e.getMessage());
		break;
	    }
	}
    }
}
