/*
 * DifferentialCheckImage.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.diff;

import java.awt.*;
import java.io.*;
import java.util.Vector;
import misao.util.*;
import misao.pixy.*;

public class DifferentialCheckImage {
    private DifferentialPixyOutputData output_data;

    private MinMaxAverage mma1;
    private MinMaxAverage mma2;

    private Vector pair_list;
    private int pair_list_index;

    private static final int fontsize = 12;
    private static final int fontspace = 4;

    public DifferentialCheckImage ( DifferentialPixyOutputData _output_data ) {
	output_data = _output_data;
    }

    public int getFontSize ( ) {
	return fontsize;
    }

    public int getFontSpace ( ) {
	return fontspace;
    }

    public Size getImageSize ( ) {
	return new Size(201, 100);
    }

    public Size getGallerySize ( ) {
	getFirstPairData();
	int total_count = pair_list.size();
	pair_list = null;
	return new Size(201, total_count * (100 + fontsize + fontspace) + total_count - 1);
    }

    public void prepareDrawImage ( ) {
	mma1 = output_data.image1.calculateMinMaxAverage();
	mma2 = output_data.image2.calculateMinMaxAverage();
    }

    public double getMaxValue ( ) {
	return mma2.getMax();
    }

    public PairData getFirstPairData ( ) {
	pair_list = new Vector();
	pair_list_index = 0;

	Vector l = output_data.pair_list.getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);

	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		if (pair.getType() != PairType.PS) {
		    if (pair.isIllegal() == false)
			pair_list.addElement(pair);
		}
	    }
	}

	return getNextPairData();
    }

    public PairData getNextPairData ( ) {
	if (pair_list_index < pair_list.size()) {
	    PairData pair = (PairData)pair_list.elementAt(pair_list_index);
	    pair_list_index++;
	    return pair;
	}
	return null;
    }

    public void createCheckImage ( String dir ) throws Exception {
	prepareDrawImage();
	PairData pair = getFirstPairData();
	while (pair != null) {
	    ImageGray image = new ImageGray(getImageSize());
	    drawImage(image, pair, 0, 0);

	    File file = new File(dir + "/" + pair.getName() + ".fts");
	    ImageFileFits fits = new ImageFileFits(image);
	    fits.save(file);

	    pair = getNextPairData();
	}
    }

    public void drawImage ( ImageGray image, PairData pair, int x0, int y0 ) throws Exception {
	StarData sd = pair.getFirstStarData();
	int base_x = (int)sd.getX();
	int base_y = (int)sd.getY();
	Point pt = new Point(0,0);
	Point pt2 = new Point(0,0);
	for (pt.y = base_y - 50 ; pt.y < base_y + 50 ; pt.y++, pt2.y++) {
	    for (pt.x = base_x - 50, pt2.x = 0 ; pt.x < base_x + 50 ; pt.x++, pt2.x++) {
		// rorate image1 to fit image2
		Point pt3 = new Point(pt2.x, pt2.y);
		if (Math.abs(output_data.map_function.getTheta() - 180) <= 45) {
		    pt3.x = 99 - pt2.x;
		    pt3.y = 99 - pt2.y;
		} else if (Math.abs(output_data.map_function.getTheta() - 90) <= 45) {
		    pt3.x = 99 - pt2.y;
		    pt3.y = pt2.x;
		} else if (Math.abs(output_data.map_function.getTheta() - 270) <= 45) {
		    pt3.x = pt2.x;
		    pt3.y = 99 - pt2.x;
		}

		// shift value of image1 to fit image2
		image.setValue(new Point(pt3.x + x0, pt3.y + y0), output_data.image1.getValue(pt) + (mma2.getAverage() - mma1.getAverage()));
	    }
	}

	sd = pair.getSecondStarData();
	base_x = (int)sd.getX();
	base_y = (int)sd.getY();
	pt = new Point(0,0);
	pt2 = new Point(101,0);
	for (pt.y = base_y - 50 ; pt.y < base_y + 50 ; pt.y++, pt2.y++) {
	    for (pt.x = base_x - 50, pt2.x = 101 ; pt.x < base_x + 50 ; pt.x++, pt2.x++) {
		image.setValue(new Point(pt2.x + x0, pt2.y + y0), output_data.image2.getValue(pt));
	    }
	}
    }
}
