/*
 * Identifier.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.host;

import java.io.*;
import java.util.*;
import misao.pixy.*;
import misao.util.*;

public class Identifier {
    private PixyPairList pair_list;
    private ImageInformation image_info;

    private boolean output_to_console;
    private String check_file;

    private static final int MODE_ONLY_NP = 0;
    private static final int MODE_ONLY_NPRM = 1;
    private static final int MODE_ALL_PS = 2;
    private static final int MODE_ALSO_NR = 3;
    private int mode;

    public Identifier ( PixyPairList _pair_list, ImageInformation _image_info ) {
	pair_list = _pair_list;
	image_info = _image_info;

	output_to_console = true;
	check_file = null;

	mode = MODE_ALSO_NR;
    }

    public void outputToConsole ( boolean f ) {
	output_to_console = f;
    }

    public void outputToCheckFile ( String filename ) {
	check_file = filename;
    }

    public void setPrintOnlyNonpair ( ) {
	mode = MODE_ONLY_NP;
    }
    public void setPrintOnlyRemarkable ( ) {
	mode = MODE_ONLY_NPRM;
    }
    public void setPrintAllPair ( ) {
	mode = MODE_ALL_PS;
    }
    public void setPrintNR ( ) {
	mode = MODE_ALSO_NR;
    }

    public PixyPairList operate ( ) throws Exception {
	PrintManager print_manager = new PrintManager();
	print_manager.outputToConsole(output_to_console);
	if (check_file != null  &&  check_file.length() > 0)
	    print_manager.openFile(check_file);

	print_manager.println("[Identification Check List]");

	// check double identification
	Hashtable hash = new Hashtable();

	ReferenceCatalogBaseManager ref_catalog_manager = new ReferenceCatalogBaseManager();

	double fov = image_info.field_of_view / 2.0 * 1.5;
	if (image_info.image_size.getWidth() < image_info.image_size.getHeight())
	    fov *= (double)image_info.image_size.getHeight() / (double)image_info.image_size.getWidth();
	double r_max = (image_info.pos_error.getRA() + image_info.pos_error.getDecl()) / 2.0 * 3.0;
	double pixel_size = image_info.field_of_view / (double)image_info.image_size.getWidth() * 3600.0;
	if (r_max < 90.0)
	    r_max = 90.0;
	if (r_max < pixel_size * 1.414)
	    r_max = pixel_size * 1.414;
	if (r_max > 600.0)
	    r_max = 600.0;
	r_max /= 3600.0;

	Vector id_list = new Vector();

	Vector list = pair_list.getAllList();
	for (int k = 0 ; k < list.size() ; k++) {
	    Vector l = (Vector)list.elementAt(k);

	    for (int j = 0 ; j < l.size() ; j++) {
		PixyPairData pair = (PixyPairData)l.elementAt(j);
		DetectData sd_detect = (DetectData)pair.getFirstStarData();
		CatalogData sd_catalog = (CatalogData)pair.getSecondStarData();

		if (mode == MODE_ONLY_NP  &&  (pair.getType() == PairType.PS  ||  pair.getType() == PairType.RM  ||  pair.getType() == PairType.RP))
		    continue;
		if (mode == MODE_ONLY_NPRM  &&  pair.getType() == PairType.PS)
		    continue;

		Coor coor = new Coor();
		if (sd_catalog != null)
		    coor = sd_catalog.getCoor();
		else
		    coor = sd_detect.getCoor();

		Vector ref_list = ref_catalog_manager.findData(coor, r_max);
		for (int i = 0 ; i < ref_list.size() ; i++) {
		    ReferenceData data = (ReferenceData)ref_list.elementAt(i);
		    double r_min = data.getCoor().angular_distance(coor) * 3600.0;

		    String r_detect = "";
		    if (sd_detect != null  &&  (pair.getType() != PairType.NC  &&  pair.getType() != PairType.NR)) {
			double r = data.getCoor().angular_distance(sd_detect.getCoor()) * 3600.0;
			r_detect += Astro.toString(r, 7, 5).trim() + "\"  ";
			if (r < r_min)
			    r_min = r;
			StarData sd = sd_detect.getFirstAppendix();
			while (sd != null) {
			    r = data.getCoor().angular_distance(sd.getCoor()) * 3600.0;
			    r_detect += Astro.toString(r, 7, 5).trim() + "\"  ";
			    if (r < r_min)
				r_min = r;
			    sd = sd_detect.getNextAppendix();
			}
		    }
		    String r_catalog = "";
		    if (sd_catalog != null) {
			double r = data.getCoor().angular_distance(sd_catalog.getCoor()) * 3600.0;
			r_catalog += Astro.toString(r, 7, 5).trim() + "\"  ";
			if (r < r_min)
			    r_min = r;
			StarData sd = sd_catalog.getFirstAppendix();
			while (sd != null) {
			    r = data.getCoor().angular_distance(sd.getCoor()) * 3600.0;
			    r_catalog += Astro.toString(r, 7, 5).trim() + "\"  ";
			    if (r < r_min)
				r_min = r;
			    sd = sd_catalog.getNextAppendix();
			}
		    }

		    double R = data.getPositionError();
		    if (sd_catalog == null) {
			if (R < pixel_size * 1.414)
			    R = pixel_size * 1.414;
		    }

		    if (r_min < R) {
			// identified
			pair.addAlias(new ReferenceName(data.getName(), data.getType()));
			pair.addReference(data.getRecord());

			// check double identification
			if (hash.containsKey(data.getName())) {
			    int count = ((Integer)hash.get(data.getName())).intValue();
			    hash.remove(data.getName());
			    hash.put(data.getName(), new Integer(count + 1));
			} else {
			    hash.put(data.getName(), new Integer(1));
			}

			// check list record
			String record = "";
			String s = data.getName() + "                        ";
			record += s.substring(0,24);
			s = pair.getName() + "                ";
			record += s.substring(0,16);
			s = r_detect + "                ";
			record += s.substring(0,16);
			record += r_catalog;
			id_list.addElement(record);

			// DBVS observation data
			Dbvs dbvs = new Dbvs(MisaoProperty.getDbvsPath());
			DbvsData dbvs_data = dbvs.find(data.getDbvsName());
			if (dbvs_data != null)
			    pair.addReference(data.getDbvsName() + "  " + dbvs_data.toString());
		    }
		}
	    }
	}

	// print in order of known variable's name
	id_list = Astro.sortStringList(id_list);
	for (int i = 0 ; i < id_list.size() ; i++)
	    print_manager.println((String)id_list.elementAt(i));

	// Not identified data
	if (mode == MODE_ALSO_NR) {
	    double deg_per_pixel = (double)image_info.image_size.getWidth() / image_info.field_of_view;
	    MapFunction mf = new MapFunction(1.0, image_info.pa, new Position(0,0));

	    ReferenceCatalogBase ref_catalog = new ReferenceCatalogBase();
	    ReferenceData data = ref_catalog.findFirstData(image_info.center_coor, image_info.field_of_view / 2.0 * 1.5);
	    while (data != null) {
		if (data.getType().isMagnitudeObservationTarget()) {
		    Position pos = new CoorMapping(data.getCoor()).mappingXY(image_info.center_coor);
		    pos.rescale(deg_per_pixel);
		    pos = mf.map(pos);

		    Position df_pos = image_info.distortion_field.getValue(pos);
		    df_pos.rescale(-1.0);
		    pos.add(df_pos);
		    pos.add(new Position((double)image_info.image_size.getWidth() / 2.0, (double)image_info.image_size.getHeight() / 2.0));
		    if (0.0 <= pos.getX()  &&  pos.getX() <= (double)image_info.image_size.getWidth()  &&  0.0 <= pos.getY()  &&  pos.getY() <= (double)image_info.image_size.getHeight()) {
			if (! hash.containsKey(data.getName())) {
			    PixyPairData pair = new PixyPairData(new DetectData(pos), null, PairType.NR);
			    pair_list.addNonpairReference(pair);
			    pair.addAlias(new ReferenceName(data.getName(), data.getType()));
			    pair.addReference(data.getRecord());

			    // DBVS observation data
			    Dbvs dbvs = new Dbvs(MisaoProperty.getDbvsPath());
			    DbvsData dbvs_data = dbvs.find(data.getDbvsName());
			    if (dbvs_data != null)
				pair.addReference(data.getDbvsName() + "  " + dbvs_data.toString());
			}
		    }
		}

		data = ref_catalog.findNextData(image_info.center_coor, image_info.field_of_view / 2.0 * 1.5);
	    }

	    pair_list.setNRNames();
	}

	// check double identification
	print_manager.println("");
	print_manager.println("[Double Identifications]");
	for (Enumeration e = hash.keys() ; e.hasMoreElements() ; ) {
	    String name = (String)e.nextElement();
	    int count = ((Integer)hash.get(name)).intValue();
	    if (count > 1) {
		String s = name + "                ";
		print_manager.println(s.substring(0,16) + count);
	    }
	}

	print_manager.close();

	return pair_list;
    }
}
