/*
 * CatalogManager.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import misao.util.*;

public class CatalogManager {
    private PixyInputData input_data;
    private boolean throw_exception_at_no_cdrom;

    public CatalogManager ( PixyInputData data ) {
	input_data = data;
	throw_exception_at_no_cdrom = false;
    }

    public void setExceptionThrowableAtNoCdrom ( boolean f ) {
	throw_exception_at_no_cdrom = f;
    }

    public CatalogList loadCatalog ( Coor center_coor, double fov, double mag_limit ) throws Exception {
	if (input_data.detail_star_data)
	    return loadCatalog(2, center_coor, fov, mag_limit);
	else
	    return loadCatalog(1, center_coor, fov, mag_limit);
    }

    public CatalogList loadCatalogNull ( Coor center_coor, double fov, double mag_limit ) throws Exception {
	return loadCatalog(0, center_coor, fov, mag_limit);
    }

    private CatalogList loadCatalog ( int flag, Coor center_coor, double fov, double mag_limit ) throws Exception {
	CatalogList catalog_list = input_data.catalog.createCatalogList();

	switch (flag) {
	case 0: catalog_list.setModeNull(); break;
	case 1: catalog_list.setModeSimple(); break;
	case 2: catalog_list.setModeDetail(); break;
	}

	if (input_data.catalog.isCdrom()) {
	    boolean no_cdrom = true;
	    String no_cdrom_message = null;

	    while (no_cdrom) {
		try {
		    ((CdromList)catalog_list).load(center_coor, fov, mag_limit, input_data.root_dir);
		    no_cdrom = false;
		} catch ( CdromNotFoundException e ) {
		    if (throw_exception_at_no_cdrom)
			throw e;

		    no_cdrom_message = e.getMessage();
		}

		if (no_cdrom) {
		    System.out.println(no_cdrom_message);
		    DataInputStream system_in = new DataInputStream(System.in);
		    system_in.readLine();
		}
	    }
	} else {
	    ((CgiList)catalog_list).load(center_coor, fov, mag_limit);
	}

	input_data.catalog.unifyDuplication(catalog_list);

	return catalog_list;
    }
}
