/*
 * DistortionField.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import misao.util.*;

public class DistortionField implements Printable {
    // ideal (x,y) = real (x,y) + (dx,dy)
    private Cubics cubics_dx;
    private Cubics cubics_dy;

    private Size size;
    private boolean print_in_detail;

    public DistortionField ( ) {
	cubics_dx = new Cubics();
	cubics_dy = new Cubics();

	size = new Size(1,1);
	print_in_detail = false;
    }

    public DistortionField ( PairList pair_list, MapFunction mf ) {
	this();

	try {
	    MapFunction inv_mf = mf.inverse();

	    for (int i = 0 ; i < pair_list.getPairCount() ; i++) {
		PairData pair = pair_list.elementPairAt(i);
		StarData sd_detect = pair.getFirstStarData();
		StarData sd_catalog = pair.getSecondStarData();

		Position pos = inv_mf.map(sd_catalog.getPosition());

		cubics_dx.setData(sd_detect.getX(), sd_detect.getY(), pos.getX() - sd_detect.getX());
		cubics_dy.setData(sd_detect.getX(), sd_detect.getY(), pos.getY() - sd_detect.getY());
	    }

	    cubics_dx.solve();
	    cubics_dy.solve();
	} catch (Exception e) {
	}
    }

    public DistortionField ( String str_x, String str_y ) {
	cubics_dx = new Cubics(str_x);
	cubics_dy = new Cubics(str_y);
    }

    public DistortionField inverse ( ) {
	DistortionField df = new DistortionField();
	df.cubics_dx = cubics_dx.negative();
	df.cubics_dy = cubics_dy.negative();
	return df;
    }

    public Position getValue ( Position pos ) {
	return new Position(cubics_dx.getValue(pos.getX(), pos.getY()), cubics_dy.getValue(pos.getX(), pos.getY()));
    }

    public Position getLargestValue ( ) {
	double val = 0.0;
	Position pos = new Position();
	Position p = getValue(new Position(- (double)size.getWidth() / 2.0, - (double)size.getHeight() / 2.0));
	double v = p.getX() * p.getX() + p.getY() + p.getY();
	if (v > val) {
	    val = v;
	    pos = p;
	}
	p = getValue(new Position(- (double)size.getWidth() / 2.0, (double)size.getHeight() / 2.0));
	v = p.getX() * p.getX() + p.getY() + p.getY();
	if (v > val) {
	    val = v;
	    pos = p;
	}
	p = getValue(new Position((double)size.getWidth() / 2.0, - (double)size.getHeight() / 2.0));
	v = p.getX() * p.getX() + p.getY() + p.getY();
	if (v > val) {
	    val = v;
	    pos = p;
	}
	p = getValue(new Position((double)size.getWidth() / 2.0, (double)size.getHeight() / 2.0));
	v = p.getX() * p.getX() + p.getY() + p.getY();
	if (v > val) {
	    val = v;
	    pos = p;
	}
	p = getValue(new Position());
	v = p.getX() * p.getX() + p.getY() + p.getY();
	if (v > val) {
	    val = v;
	    pos = p;
	}

	return pos;
    }

    // to be invoked before print()
    public void setPrintSize ( Size s ) {
	size = s;
    }

    public void setPrintInDetail ( boolean f ) {
	print_in_detail = f;
    }

    public void print ( PrintStream ps ) {
	if (print_in_detail)
	    printDetail(ps);
	else
	    printMap(ps);
    }

    public void printMap ( PrintStream ps ) {
	Position d_pos = getValue(new Position(- (double)size.getWidth() / 2.0, - (double)size.getHeight() / 2.0));
	ps.print("  " + Astro.toString(d_pos.getX(), 4, 2) + "," + Astro.toString(d_pos.getY(), 4, 2) + "         ");
	d_pos = getValue(new Position((double)size.getWidth() / 2.0, - (double)size.getHeight() / 2.0));
	ps.println(Astro.toString(d_pos.getX(), 4, 2) + "," + Astro.toString(d_pos.getY(), 4, 2));
	d_pos = getValue(new Position());
	ps.println("           " + Astro.toString(d_pos.getX(), 4, 2) + "," + Astro.toString(d_pos.getY(), 4, 2));
	d_pos = getValue(new Position(- (double)size.getWidth() / 2.0, (double)size.getHeight() / 2.0));
	ps.print("  " + Astro.toString(d_pos.getX(), 4, 2) + "," + Astro.toString(d_pos.getY(), 4, 2) + "         ");
	d_pos = getValue(new Position((double)size.getWidth() / 2.0, (double)size.getHeight() / 2.0));
	ps.println(Astro.toString(d_pos.getX(), 4, 2) + "," + Astro.toString(d_pos.getY(), 4, 2));
    }

    public void printDetail ( PrintStream ps ) {
	ps.println(cubics_dx.toString("dx"));
	ps.println(cubics_dy.toString("dy"));
    }
}
