/*
 * MagError.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import java.util.*;
import misao.util.*;

public class MagError implements Printable {
    double constant_dmag;

    boolean dependent_error;
    double dependent_mag0;

    public MagError ( double dm ) {
	constant_dmag = dm;
	dependent_error = false;
    }

    // elements of the list must be PixyPairData
    public MagError ( Vector list, double _constant_dmag ) throws Exception {
	constant_dmag = _constant_dmag;

	int count = 0;
	int min_M = 9999;
	int max_M = -9999;
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    StarData sd_detect = pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    double dmag = Math.abs(sd_detect.getMag() - sd_catalog.getMag()) - constant_dmag;
	    if (dmag >= 0.1) {
		double mag0 = sd_catalog.getMag() - 2.0 * Math.log(dmag) / Math.log(Astro.MAG_STEP);
		count++;

		int M = (int)(mag0 * 10.0);
		if (min_M > M)
		    min_M = M;
		if (max_M < M)
		    max_M = M;
	    }
	}
	if (count == 0)
	    return;

	dependent_error = true;

	int max_count = -1;
	int max_M0 = 0;
	int[] mag0_count = new int[max_M - min_M + 1];
	for (int M = min_M ; M <= max_M ; M++)
	    mag0_count[M - min_M] = 0;
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    StarData sd_detect = pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    double dmag = Math.abs(sd_detect.getMag() - sd_catalog.getMag()) - constant_dmag;
	    if (dmag >= 0.1) {
		double mag0 = sd_catalog.getMag() - 2.0 * Math.log(dmag) / Math.log(Astro.MAG_STEP);
		int M = (int)(mag0 * 10.0);
		mag0_count[M - min_M]++;

		if (max_count < mag0_count[M - min_M]) {
		    max_count = mag0_count[M - min_M];
		    max_M0 = M;
		}
	    }
	}
	if (max_count == 1) {
	    dependent_mag0 = (double)max_M0 / 10.0 + 0.1;
	    return;
	}

	double mag0 = (double)max_M0 / 10.0;
	int oldM0 = max_M0;
	int dM = 20;
	boolean loop = true;
	while (loop) {
	    int min_M2 = min_M;
	    if (min_M2 < max_M0 - dM)
		min_M2 = max_M0 - dM;

	    boolean loop2 = true;
	    while (loop2) {
		int loop2_count = 0;
		double loop2_avr = 0.0;
		for (int M = min_M2 ; M <= max_M0 ; M++) {
		    loop2_avr += (double)mag0_count[M - min_M];
		    loop2_count++;
		}

		if (min_M2 == min_M)
		    loop2 = false;
		else if (loop2_count == 0)
		    loop2 = false;
		else {
		    loop2_avr /= (double)loop2_count;
		    if (loop2_avr < (double)max_count / 3.0)
			loop2 = false;
		    else
			min_M2--;
		}
	    }

	    SimultaneousEquation se = new SimultaneousEquation(2);
	    for (int M = min_M2 ; M <= max_M0 ; M++) {
		se.coef[0][0] += 1.0;
		se.coef[0][1] += (double)M;
		se.coef[0][2] -= (double)mag0_count[M - min_M];
		se.coef[1][0] += (double)M;
		se.coef[1][1] += (double)M * (double)M;
		se.coef[1][2] -= (double)M * (double)mag0_count[M - min_M];
	    }
	    se.solve();
	    double a = se.getAnswer(0);
	    double b = se.getAnswer(1);

	    mag0 = - a / b / 10.0;
	    int M0 = (int)(mag0 * 10.0);

	    if (M0 >= oldM0)
		loop = false;
	    else {
		dM = (int)(max_M0 - M0) + 5;
		oldM0 = M0;
	    }
	}

	dependent_mag0 = mag0;
    }

    public static MagError createMagError ( String s ) {
	if (s.indexOf("+/-") < 0) {
	    int p = s.indexOf(' ');
	    if (p >= 0)
		s = s.substring(0, p);
	    return new MagError(Double.valueOf(s).doubleValue());
	}

	StringTokenizer st = new StringTokenizer(s);
	st.nextToken();		// +/-
	MagError err = new MagError(Double.valueOf(st.nextToken()).doubleValue());
	st.nextToken();		// mag
	st.nextToken();		// +/-
	st.nextToken();		// 1.0
	st.nextToken();		// at
	err.dependent_error = true;
	try {
	    err.dependent_mag0 = Double.valueOf(st.nextToken()).doubleValue();
	} catch ( NumberFormatException e ) {
	    err.dependent_mag0 = 0.0;
	}
	return err;
    }

    // deprecated
    public double getDeltaMag ( ) {
	return constant_dmag;
    }

    public double getConstantError ( ) {
	return constant_dmag;
    }

    public double getDependentError ( double mag ) {
	if (dependent_error)
	    return Math.pow(Astro.MAG_STEP, (mag - dependent_mag0) / 2.0);

	return Astro.ILLEGAL_MAG;
    }

    public double getRemarkableThreshold ( double mag ) {
	if (dependent_error)
	    return constant_dmag + Math.pow(Astro.MAG_STEP, (mag - dependent_mag0) / 2.0);

	return constant_dmag * 3.0;
    }

    public boolean isBrightnessDependent ( ) {
	return dependent_error;
    }

    public String toString ( ) {
	if (dependent_error)
	    return toStringConstantError() + " " + toStringDependentError();

	return Astro.toString(constant_dmag, 5, 2).trim() + " mag";
    }

    public String toStringConstantError ( ) {
	return "+/- " + Astro.toString(constant_dmag, 5, 2).trim() + " mag";
    }

    public String toStringDependentError ( ) {
	return "+/- 1.0 at " + Astro.toString(dependent_mag0, 5, 2).trim() + " mag";
    }

    public void print ( PrintStream ps ) {
	ps.println("constant error: " + toStringConstantError());
	if (dependent_error)
	    ps.println("dependent error: " + toStringDependentError());
    }
}
