/*
 * MagnitudeAdjustment.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import java.util.*;
import misao.util.*;

public class MagnitudeAdjustment implements Printable {
    private static final int width = 20;
    private static final double mag_width = 1.0;

    int min_M;
    int max_M;
    double[] mag_correct;

    // elements of the list must be PixyPairData
    // the list must be sorted in order of detected magnitude
    public MagnitudeAdjustment ( Vector list ) throws Exception {
	min_M = 9999;
	max_M = -9999;
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    StarData sd_catalog = pair.getSecondStarData();

	    int M = (int)(sd_catalog.getMag() * 10.0);
	    if (min_M > M)
		min_M = M;
	    if (max_M < M)
		max_M = M;
	}

	mag_correct = new double[max_M - min_M + 1];
	int old_si = 0;
	int old_ei = 0;
	for (int M = min_M ; M <= max_M ; M++) {
	    double current_mag = (double)M / 10.0;

	    int si = -1;
	    int ei = -1;
	    double faintest_mag = 0.0;
	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		StarData sd_catalog = pair.getSecondStarData();

		if (Math.abs(current_mag - sd_catalog.getMag()) <= mag_width) {
		    ei = i;
		    if (si < 0)
			si = i;
		    faintest_mag = sd_catalog.getMag();
		} else if (si >= 0)
		    break;
	    }
	    if (ei == list.size() - 1) {
		si = 0;
		for (int i = 0 ; i < list.size() ; i++) {
		    PairData pair = (PairData)list.elementAt(i);
		    StarData sd_catalog = pair.getSecondStarData();

		    if (faintest_mag - mag_width * 2.0 <= sd_catalog.getMag()) {
			si = i;
			break;
		    }
		}
	    }
	    if (si < 0) {
		si = old_si;
		ei = old_ei;
	    }
	    if (ei - si + 1 < width) {
		ei = si + width - 1;
		if (ei >= list.size()) {
		    ei = list.size() - 1;
		    si = list.size() - width;
		    if (si < 0)
			si = 0;
		}
	    }
	    old_si = si;
	    old_ei = ei;

	    SimultaneousEquation se = new SimultaneousEquation(2);
	    for (int i = si ; i <= ei ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		DetectData sd_detect = (DetectData)pair.getFirstStarData();
		StarData sd_catalog = pair.getSecondStarData();

		se.coef[0][0] += 1.0;
		se.coef[0][1] += sd_catalog.getMag();
		se.coef[0][2] -= sd_detect.getMag();
		se.coef[1][0] += sd_catalog.getMag();
		se.coef[1][1] += sd_catalog.getMag() * sd_catalog.getMag();
		se.coef[1][2] -= sd_catalog.getMag() * sd_detect.getMag();
	    }
	    se.solve();
	    double a = se.getAnswer(0);
	    double b = se.getAnswer(1);

	    int count = 0;
	    se = new SimultaneousEquation(2);
	    for (int i = si ; i <= ei ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		DetectData sd_detect = (DetectData)pair.getFirstStarData();
		StarData sd_catalog = pair.getSecondStarData();

		double dm = Math.abs((a + b * sd_catalog.getMag()) - sd_detect.getMag());
		if (dm < 1.0) {
		    count++;
		    se.coef[0][0] += 1.0;
		    se.coef[0][1] += sd_catalog.getMag();
		    se.coef[0][2] -= sd_detect.getMag();
		    se.coef[1][0] += sd_catalog.getMag();
		    se.coef[1][1] += sd_catalog.getMag() * sd_catalog.getMag();
		    se.coef[1][2] -= sd_catalog.getMag() * sd_detect.getMag();
		}
	    }
	    if (count > width / 2) {
		se.solve();
		a = se.getAnswer(0);
		b = se.getAnswer(1);
	    }

	    mag_correct[M - min_M] = current_mag - (a + b * current_mag);
	}
    }

    public double getDeltaMag ( double mag ) {
	int M = (int)(mag * 10.0);
	if (M < min_M)
	    return mag_correct[0];
	if (M > max_M)
	    return mag_correct[max_M - min_M];
	return mag_correct[M - min_M];
    }

    public void print ( PrintStream ps ) {
	ps.println("Magnitude adjustment:");
	for (int M = min_M ; M <= max_M ; M++) {
	    double current_mag = (double)M / 10.0;
	    if (M % 5 == 0) {
		ps.println("  " + Astro.toString(current_mag, 4, 2) + " -> " + Astro.toString(current_mag + getDeltaMag(current_mag), 4, 2));
	    }
	}
    }
}
