/*
 * PairData.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import java.util.*;
import misao.util.*;
import misao.host.*;

public class PairData {
    protected PairType type;
    protected String name;
    protected Vector alias_list;

    protected Object object1;	// detected star
    protected Object object2;	// catalog data

    private Vector memo_list;
    private Vector reference_list;

    private boolean illegal_flag;

    protected PairData ( ) {
	name = "";
	alias_list = new Vector();
	memo_list = new Vector();
	reference_list = new Vector();

	illegal_flag = false;
    }

    public PairData ( StarData sd1, StarData sd2 ) {
	this(sd1, sd2, PairType.PS);
    }
    public PairData ( StarData sd1, StarData sd2, PairType t ) {
	this();

	object1 = (Object)sd1;
	object2 = (Object)sd2;
	type = t;
    }

    public PairData ( PairData pair1, PairData pair2 ) {
	this(pair1, pair2, PairType.PS);
    }
    public PairData ( PairData pair1, PairData pair2, PairType t ) {
	this();

	object1 = (Object)pair1;
	object2 = (Object)pair2;
	type = t;
    }

    public void setType ( PairType t ) {
	type = t;
    }

    public PairType getType ( ) {
	return type;
    }

    public void setName ( String s ) {
	name = s;
    }

    public String getName ( ) {
	return name;
    }

    public void setIllegal ( ) {
	illegal_flag = true;
    }

    public boolean isIllegal ( ) {
	return illegal_flag;
    }

    public boolean isBlending ( ) {
	String first_object = "";
	for (int i = 0 ; i < alias_list.size() ; i++) {
	    ReferenceName n = (ReferenceName)alias_list.elementAt(i);
	    if (n.getType() == ReferenceType.REF_O) {
		if (first_object.length() == 0)
		    first_object = n.getName();
		else if (! first_object.equals(n.getName()))
		    return true;
	    }
	}
	return false;
    }

    public void addAlias ( ReferenceName alias ) {
	for (int i = 0 ; i < alias_list.size() ; i++) {
	    ReferenceName n = (ReferenceName)alias_list.elementAt(i);
	    if (alias.getName().equals(n.getName()))
		return;
	}

	alias_list.addElement(alias);
    }

    public boolean hasReference ( ) {
	Vector l = getAliasList();
	for (int i = 0 ; i < l.size() ; i++) {
	    ReferenceName ref_name = (ReferenceName)l.elementAt(i);
	    if (ref_name.getType() == ReferenceType.REF_V  ||  ref_name.getType() == ReferenceType.REF_S  ||  ref_name.getType() == ReferenceType.REF_E  ||  ref_name.getType() == ReferenceType.REF_N)
		return true;
	}
	return false;
    }

    public Vector getAliasList ( ) {
	Vector new_list = new Vector();
	for (int i = 0 ; i < alias_list.size() ; i++)
	    new_list.addElement(alias_list.elementAt(i));
	return new_list;
    }

    public void addMemo ( String s ) {
	for (int i = 0 ; i < memo_list.size() ; i++) {
	    if (((String)memo_list.elementAt(i)).equals(s))
		return;
	}
	memo_list.addElement(s);
    }

    public void addReference ( String s ) {
	for (int i = 0 ; i < reference_list.size() ; i++) {
	    if (((String)reference_list.elementAt(i)).equals(s))
		return;
	}
	reference_list.addElement(s);
    }

    public Object getFirstData ( ) {
	return object1;
    }

    public Object getSecondData ( ) {
	return object2;
    }

    public StarData getFirstStarData ( ) throws Exception {
	if (object1 == null)
	    return null;

	if (object1 instanceof StarData) {
	    return (StarData)object1;
	}

	throw new Exception();
    }
    public StarData getSecondStarData ( ) throws Exception {
	if (object2 == null)
	    return null;

	if (object2 instanceof StarData) {
	    return (StarData)object2;
	}

	throw new Exception();
    }

    public PairData getFirstPairData ( ) throws Exception {
	if (object1 == null)
	    return null;

	if (object1 instanceof PairData) {
	    return (PairData)object1;
	}

	throw new Exception();
    }
    public PairData getSecondPairData ( ) throws Exception {
	if (object2 == null)
	    return null;

	if (object2 instanceof PairData) {
	    return (PairData)object2;
	}

	throw new Exception();
    }

    public double getFirstMag ( ) {
	if (object1 instanceof StarData)
	    return ((StarData)object1).getMag();
	if (object1 instanceof PairData)
	    return ((PairData)object1).getBrightestMag();
	return Astro.ILLEGAL_MAG;
    }
    public double getSecondMag ( ) {
	if (object1 instanceof StarData)
	    return ((StarData)object2).getMag();
	if (object1 instanceof PairData)
	    return ((PairData)object2).getBrightestMag();
	return Astro.ILLEGAL_MAG;
    }

    public double getBrightestMag ( ) {
	double mag = Astro.ILLEGAL_MAG;
	if (object1 instanceof StarData) {
	    if (mag > ((StarData)object1).getMag())
		mag = ((StarData)object1).getMag();
	}
	if (object1 instanceof PairData) {
	    double m = ((PairData)object1).getBrightestMag();
	    if (m != Astro.ILLEGAL_MAG  &&  mag > m)
		mag = m;
	}
	if (object2 instanceof StarData) {
	    if (mag > ((StarData)object2).getMag())
		mag = ((StarData)object2).getMag();
	}
	if (object2 instanceof PairData) {
	    double m = ((PairData)object2).getBrightestMag();
	    if (m != Astro.ILLEGAL_MAG  &&  mag > m)
		mag = m;
	}
	return mag;
    }

    protected String getFirstHeader ( ) {
	return "1|";
    }
    protected String getSecondHeader ( ) {
	return "2|";
    }

    public void print ( PrintStream ps ) {
	print(ps, "");
    }

    public void print ( PrintStream ps, String header ) {
	try {
	    if (object1 instanceof StarData) {
		StarData sd0 = getFirstStarData();
		ps.println(header + getFirstHeader() + " " + sd0.getRecord());
		StarData sd = sd0.getFirstAppendix();
		while (sd != null) {
		    ps.println(header + getFirstHeader() + " " + sd.getRecord());
		    sd = sd0.getNextAppendix();
		}
	    } else if (object1 instanceof PairData) {
		PairData pair = getFirstPairData();
		pair.print(ps, header + getFirstHeader());
	    }
	    if (object2 instanceof StarData) {
		StarData sd0 = getSecondStarData();
		ps.println(header + getSecondHeader() + " " + sd0.getRecord());
		StarData sd = sd0.getFirstAppendix();
		while (sd != null) {
		    ps.println(header + getSecondHeader() + " " + sd.getRecord());
		    sd = sd0.getNextAppendix();
		}
	    } else if (object2 instanceof PairData) {
		PairData pair = getSecondPairData();
		pair.print(ps, header + getSecondHeader());
	    }

	    for (int i = 0 ; i < reference_list.size() ; i++) {
		String s = (String)reference_list.elementAt(i);
		ps.println(header + "R| " + s);
	    }

	    for (int i = 0 ; i < memo_list.size() ; i++) {
		String s = (String)memo_list.elementAt(i);
		ps.println(header + "M| " + s);
	    }
	} catch (Exception e) {
	}
    }

    public static PairData createPairData ( String[] record ) {
	PairData pair = new PairData();

	pair.type = PairType.PS;

	Vector pair1_list = new Vector();
	Vector pair2_list = new Vector();

	for (int i = 0 ; i < record.length ; i++) {
	    if (record[i].length() >= 2  &&  record[i].charAt(1) == '|') {
		switch (record[i].charAt(0)) {
		case '1':
		    pair1_list.addElement(record[i].substring(2));

		    if (pair.type == PairType.NP)
			pair.type = PairType.NP1;
		    break;
		case '2':
		    pair2_list.addElement(record[i].substring(2));

		    if (pair.type == PairType.NP)
			pair.type = PairType.NP2;
		    break;
		case 'R':
		    pair.addReference(record[i].substring(3));
		    break;
		case 'M':
		    pair.addMemo(record[i].substring(3));
		    break;
		default:
		    return null;
		}
	    } else if (record[i].length() >= 2  &&  record[i].charAt(1) == '=') {
		pair.addAlias(new ReferenceName(record[i].substring(2), ReferenceType.getType(record[i].charAt(0))));
	    } else {
		pair.setName(record[i]);

		if (pair.name.substring(0,3).equals("PS-"))
		    pair.type = PairType.PS;
		else if (pair.name.substring(0,3).equals("RM-"))
		    pair.type = PairType.RM;
		else if (pair.name.substring(0,3).equals("RP-"))
		    pair.type = PairType.RP;
		else if (pair.name.substring(0,3).equals("NP-"))
		    pair.type = PairType.NP;
		else if (pair.name.substring(0,3).equals("NR-"))
		    pair.type = PairType.NR;
	    }
	}

	pair.object1 = null;
	pair.object2 = null;

	if (pair1_list.size() > 0) {
	    String[] string_list = new String[pair1_list.size()];
	    for (int i = 0 ; i < pair1_list.size() ; i++)
		string_list[i] = (String)pair1_list.elementAt(i);
	    PairData pair1 = createPairData(string_list);
	    if (pair1 == null)
		pair1 = PixyPairData.createPairData(string_list);
	    pair.object1 = pair1;
	}
	if (pair2_list.size() > 0) {
	    String[] string_list = new String[pair2_list.size()];
	    for (int i = 0 ; i < pair2_list.size() ; i++)
		string_list[i] = (String)pair2_list.elementAt(i);
	    PairData pair2 = createPairData(string_list);
	    if (pair2 == null)
		pair2 = PixyPairData.createPairData(string_list);
	    pair.object2 = pair2;
	}

	return pair;
    }
}
