/*
 * PhotometricReporter.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import java.util.*;
import misao.util.*;
import misao.host.*;

public class PhotometricReporter {
    private PixyPairList pair_list;
    private ImageInformation image_info;

    private boolean output_to_console;
    private String report_file;

    public PhotometricReporter ( PixyPairList _pair_list, ImageInformation _image_info ) {
	pair_list = _pair_list;
	image_info = _image_info;

	output_to_console = true;
	report_file = null;
    }

    public void outputToConsole ( boolean f ) {
	output_to_console = f;
    }

    public void outputToReportFile ( String filename ) {
	report_file = filename;
    }

    public void operate ( ) throws Exception {
	PrintManager print_manager = new PrintManager();
	print_manager.outputToConsole(output_to_console);
	if (report_file != null  &&  report_file.length() > 0)
	    print_manager.openFile(report_file);

	Vector list = pair_list.getAllList();
	Vector output_list = new Vector();

	for (int i = 0 ; i < list.size() ; i++) {
	    Vector l = (Vector)list.elementAt(i);

	    for (int j = 0 ; j < l.size() ; j++) {
		PixyPairData data = (PixyPairData)l.elementAt(j);
		DetectData sd_detect = (DetectData)data.getFirstData();

		String record = createRecord(sd_detect, data);
		output_list.addElement(record);

		DetectData sd = (DetectData)sd_detect.getFirstAppendix();
		while (sd != null) {
		    record = createRecord(sd, data);
		    output_list.addElement(record);
		    sd = (DetectData)sd_detect.getNextAppendix();
		}
	    }
	}

	output_list = Astro.sortStringList(output_list);
	for (int i = 0 ; i < output_list.size() ; i++) {
	    String s = (String)output_list.elementAt(i);

	    // the first character is just a priority for sort
	    print_manager.println(s.substring(1));
	}

	print_manager.close();
    }

    private String createRecord ( DetectData sd, PixyPairData data ) {
	Vector alias_list = data.getAliasList();

	/*
	 *  Sort order:
	 *  0: [code V]
	 *  1: [code S]
	 *  2: [code E]
	 *  3: [code N]
	 *  4: J012345.67+012345.6
	 *  5: [code O]
	 *  6: blending
	 */

	String name = "";
	if (alias_list == null  ||  alias_list.size() == 0) {
	    String s = sd.getCoor().toStringRA() + sd.getCoor().toStringDecl();
	    name = "4J";
	    for (int j = 0 ; j < s.length() ; j++) {
		if (s.charAt(j) != ' ')
		    name += s.charAt(j);
	    }
	} else if (data.isBlending()) {
	    for (int j = 0 ; j < alias_list.size() ; j++) {
		ReferenceName n = (ReferenceName)alias_list.elementAt(j);

		if (n.getType() == ReferenceType.REF_O) {
		    if (name.length() > 0)
			name += " + ";
		    name += n.getName();
		    alias_list.removeElementAt(j);
		    j--;
		}
	    }
	    name = "6" + name;
	} else {
	    int priority = 9;
	    int index = -1;

	    for (int j = 0 ; j < alias_list.size() ; j++) {
		ReferenceName n = (ReferenceName)alias_list.elementAt(j);

		int p = 9;
		if (n.getType() == ReferenceType.REF_V)
		    p = 0;
		else if (n.getType() == ReferenceType.REF_S)
		    p = 1;
		else if (n.getType() == ReferenceType.REF_E)
		    p = 2;
		else if (n.getType() == ReferenceType.REF_N)
		    p = 3;
		else if (n.getType() == ReferenceType.REF_O)
		    p = 5;

		if (p < priority) {
		    priority = p;
		    index = j;
		}
	    }

	    ReferenceName n = (ReferenceName)alias_list.elementAt(index);
	    name = "" + priority + n.getName();
	    alias_list.removeElementAt(index);
	}

	String mag = "";
	if (data.getType() == PairType.NC  ||  data.getType() == PairType.NR)
	    mag = "<" + Astro.toString(image_info.limiting_mag, 4, 2).trim();
	else {
	    mag = Astro.toString(sd.getMag(), 4, 2).trim();
	    if (sd.isEdge())
		mag += ":";
	}
	mag += sd.getMagCode();

	String record = name + "\t" + mag;

	if (data.isBlending()) {
	    boolean alias_flag = false;
	    if (alias_list != null) {
		for (int j = 0 ; j < alias_list.size() ; j++) {
		    ReferenceName n = (ReferenceName)alias_list.elementAt(j);
		    if (alias_flag) {
			record += ", " + n.getName();
		    } else {
			record += "\t(" + n.getName();
			alias_flag = true;
		    }
		}
		if (alias_flag)
		    record += ")";
	    }
	} else {
	    boolean alias_flag = false;
	    if (alias_list != null) {
		for (int j = 0 ; j < alias_list.size() ; j++) {
		    ReferenceName n = (ReferenceName)alias_list.elementAt(j);
		    if (alias_flag) {
			record += " = " + n.getName();
		    } else {
			record += "\t= " + n.getName();
			alias_flag = true;
		    }
		}
	    }
	}

	return record;
    }
}
