/*
 * PixyPairList.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.Color;
import java.io.*;
import java.util.Vector;
import misao.util.*;
import misao.host.*;

public class PixyPairList extends PairList {
    // first list is detected stars list
    // second list is catalog data list

    public PixyPairList ( ) {
	super();
    }

    public PixyPairList ( PairList pl ) {
	super(pl);
    }

    public int getDetectCount ( ) {
	return getFirstListCount();
    }

    public int getCatalogCount ( ) {
	return getSecondListCount();
    }

    protected PairType getFirstNonpairType ( ) {
	return PairType.ND;
    }

    protected PairType getSecondNonpairType ( ) {
	return PairType.NC;
    }

    public void addPairElement ( StarData data1, StarData data2 ) {
	PS_list.addElement(new PixyPairData(data1, data2, PairType.PS));
    }

    public void addFirstNonpairElement ( StarData data ) {
	NP_list1.addElement(new PixyPairData(data, null, PairType.ND));
    }

    public void addSecondNonpairElement ( StarData data ) {
	NP_list2.addElement(new PixyPairData(null, data, PairType.NC));
    }

    public void appendFirstNonpairList ( StarList l ) {
	for (int i = 0 ; i < l.size() ; i++)
	    NP_list1.addElement(new PixyPairData(l.elementAt(i), null, PairType.ND));
    }

    public void appendSecondNonpairList ( StarList l ) {
	for (int i = 0 ; i < l.size() ; i++)
	    NP_list2.addElement(new PixyPairData(null, l.elementAt(i), PairType.NC));
    }

    public void setAllNames ( ) {
	int PS_count = 1;
	int RM_count = 1;
	int RP_count = 1;
	int ND_count = 1;
	int NC_count = 1;
	int NR_count = 1;

	Vector l = getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);
	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		if (pair.type.isNonpairDetected()) {
		    pair.setName("ND-" + Astro.toStringZeroPadding(ND_count++, 5));
		} else if (pair.type.isNonpairCatalog()) {
		    pair.setName("NC-" + Astro.toStringZeroPadding(NC_count++, 5));
		} else if (pair.type.isRemarkableMagnitude()) {
		    pair.setName("RM-" + Astro.toStringZeroPadding(RM_count++, 5));
		} else if (pair.type.isRemarkablePosition()) {
		    pair.setName("RP-" + Astro.toStringZeroPadding(RP_count++, 5));
		} else if (pair.type.isNonpairReference()) {
		    pair.setName("NR-" + Astro.toStringZeroPadding(NR_count++, 5));
		} else {
		    pair.setName("PS-" + Astro.toStringZeroPadding(PS_count++, 5));
		}
	    }
	}
    }

    public void setAllAlias ( ) throws Exception {
	Vector l = getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);
	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		if (! pair.type.isNonpairDetected()) {
		    CatalogData sd_catalog = (CatalogData)pair.getSecondStarData();
		    pair.addAlias(new ReferenceName(sd_catalog.getID(), ReferenceType.REF_O));
		    CatalogData sd = (CatalogData)sd_catalog.getFirstAppendix();
		    while (sd != null) {
			pair.addAlias(new ReferenceName(sd.getID(), ReferenceType.REF_O));
			sd = (CatalogData)sd_catalog.getNextAppendix();
		    }
		}
	    }
	}
    }

    public void printStatistics ( PrintStream ps ) {
	new Parameter("PS", "" + PS_list.size()).print(ps);
	new Parameter("RM", "" + RM_list.size()).print(ps);
	new Parameter("RP", "" + RP_list.size()).print(ps);
	new Parameter("ND", "" + NP_list1.size()).print(ps);
	new Parameter("NC", "" + NP_list2.size()).print(ps);
	new Parameter("NR", "" + NR_list.size()).print(ps);
    }

    // calculate magnitude translation formula
    public MagnitudeTranslationFormula calculateMagnitudeTranslationFormula ( ) throws Exception {
	return calculateMagnitudeTranslationFormula(false, 99.9);
    }
    public MagnitudeTranslationFormula calculateMagnitudeTranslationFormula ( double mag_limit ) throws Exception {
	return calculateMagnitudeTranslationFormula(false, mag_limit);
    }
    public MagnitudeTranslationFormula calculateMagnitudeTranslationFormulaInPrinciple ( ) throws Exception {
	return calculateMagnitudeTranslationFormula(true, 99.9);
    }
    public MagnitudeTranslationFormula calculateMagnitudeTranslationFormulaInPrinciple ( double mag_limit ) throws Exception {
	return calculateMagnitudeTranslationFormula(true, mag_limit);
    }
    private MagnitudeTranslationFormula calculateMagnitudeTranslationFormula ( boolean gradient_flag, double mag_limit ) throws Exception {
	SimultaneousEquation se = new SimultaneousEquation(2);

	double gradient = -1.0;

	double mag_base = 0.0;
	double weight = 1.0;
	double a = 0.0;
	double b = 0.0;
	int count = 0;
	for (int i = 0 ; i < getPairCount() ; i++) {
	    PairData pair = elementPairAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();
	    double X = Math.log(sd_detect.getValue()) / Math.log(Astro.MAG_STEP);
	    double Y = sd_catalog.getMag();

	    if (Y <= mag_limit) {
		count++;

		if (i == 0)
		    mag_base = Y;
		else
		    weight = 1.0 / Math.pow(2.152, Y - mag_base);

		if (gradient_flag) {
		    weight = 1.0;
		    a += (Y - X * gradient) * weight;
		    b += weight;
		} else {
		    se.coef[0][0] += 1.0 * weight;
		    se.coef[0][1] += X * weight;
		    se.coef[0][2] -= Y * weight;
		    se.coef[1][0] += X * weight;
		    se.coef[1][1] += X * X * weight;
		    se.coef[1][2] -= X * Y * weight;
		}
	    }
	}

	MagnitudeTranslationFormula mag_formula = new MagnitudeTranslationFormula(0.0, gradient);

	if (count > 0) {
	    if (gradient_flag) {
		a /= b;
		b = gradient;
	    } else {
		se.solve();
		a = se.getAnswer(0);
		b = se.getAnswer(1);
	    }
	}
	mag_formula = new MagnitudeTranslationFormula(a, b);

	return mag_formula;
    }

    // calculate error of magnitude
    public double calculateErrorMag ( MagnitudeTranslationFormula mag_formula ) throws Exception {
	double error_mag = 0.0;
	for (int i = 0 ; i < getPairCount() ; i++) {
	    PairData pair = elementPairAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();
	    double mag_detect = mag_formula.calculate(sd_detect.getValue());
	    double mag_catalog = sd_catalog.getMag();
	    error_mag += (mag_detect - mag_catalog) * (mag_detect - mag_catalog);
	}
	error_mag = Math.sqrt(error_mag / (double)getPairCount());
	return error_mag;
    }

    // calculate constant magnitude error (independent error on brightness)
    public double calculateConstantMagError ( int max_count ) throws Exception {
	int count = 0;
	double amt = 0.0;
	double amt2 = 0.0;
	for (int i = 0 ; i < getPairCount()  &&  i < max_count ; i++) {
	    PairData pair = elementPairAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    double dmag = sd_detect.getMag() - sd_catalog.getMag();

	    amt += dmag;
	    amt2 += dmag * dmag;
	    count++;
	}
	if (count == 0)
	    return 0.0;

	amt /= (double)count;
	amt2 /= (double)count;
	double sigma = Math.sqrt(amt2 - amt * amt);

	int count2 = 0;
	amt = 0.0;
	amt2 = 0.0;
	for (int i = 0 ; i < count ; i++) {
	    PairData pair = elementPairAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    double dmag = sd_detect.getMag() - sd_catalog.getMag();

	    if (Math.abs(dmag) < 2.0 * sigma) {
		amt += dmag;
		amt2 += dmag * dmag;
		count2++;
	    }
	}
	if (count2 > 0  &&  count2 > count / 2) {
	    amt /= (double)count2;
	    amt2 /= (double)count2;
	    sigma = Math.sqrt(amt2 - amt * amt);
	}

	return sigma;
    }

    // calculate brightness dependent magnitude error
    public MagError calculateBrightnessDependentMagError( double constant_dmag ) throws Exception {
	return new MagError(PS_list, constant_dmag);
    }

    // calculate map function and errors of R.A., Decl.
    public MapFunction calculateMapFunction ( ) throws Exception {
	return calculateMapFunction(new DistortionField());
    }
    public MapFunction calculateMapFunction ( DistortionField df ) throws Exception {
	// calculate map function
	SimultaneousEquation sex = new SimultaneousEquation(3);
	SimultaneousEquation sey = new SimultaneousEquation(3);

	for (int i = 0 ; i < getPairCount() ; i++) {
	    PairData pair = elementPairAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();
	    Position df_pos = df.getValue(sd_detect.getPosition());
	    Position pos_detect = new Position(sd_detect.getX() + df_pos.getX(), sd_detect.getY() + df_pos.getY());

	    sex.coef[0][0] += pos_detect.getX() * pos_detect.getX();
	    sex.coef[0][1] += pos_detect.getX() * pos_detect.getY();
	    sex.coef[0][2] += pos_detect.getX();
	    sex.coef[0][3] -= pos_detect.getX() * sd_catalog.getX();
	    sex.coef[1][0] += pos_detect.getY() * pos_detect.getX();
	    sex.coef[1][1] += pos_detect.getY() * pos_detect.getY();
	    sex.coef[1][2] += pos_detect.getY();
	    sex.coef[1][3] -= pos_detect.getY() * sd_catalog.getX();
	    sex.coef[2][0] += pos_detect.getX();
	    sex.coef[2][1] += pos_detect.getY();
	    sex.coef[2][2] += 1.0;
	    sex.coef[2][3] -= sd_catalog.getX();

	    sey.coef[0][0] += pos_detect.getX() * pos_detect.getX();
	    sey.coef[0][1] += pos_detect.getX() * pos_detect.getY();
	    sey.coef[0][2] += pos_detect.getX();
	    sey.coef[0][3] -= pos_detect.getX() * sd_catalog.getY();
	    sey.coef[1][0] += pos_detect.getY() * pos_detect.getX();
	    sey.coef[1][1] += pos_detect.getY() * pos_detect.getY();
	    sey.coef[1][2] += pos_detect.getY();
	    sey.coef[1][3] -= pos_detect.getY() * sd_catalog.getY();
	    sey.coef[2][0] += pos_detect.getX();
	    sey.coef[2][1] += pos_detect.getY();
	    sey.coef[2][2] += 1.0;
	    sey.coef[2][3] -= sd_catalog.getY();
	}
	sex.solve();
	sey.solve();

	// parameters of map function
	// X = fA x + fB y + fC
	// Y = fD x + fE y + fF
	double fA = sex.getAnswer(0);
	double fB = sex.getAnswer(1);
	double fC = sex.getAnswer(2);
	double fD = sey.getAnswer(0);
	double fE = sey.getAnswer(1);
	double fF = sey.getAnswer(2);

	double r = (Math.sqrt(fA * fA + fB * fB) + Math.sqrt(fD * fD + fE * fE)) / 2.0;
	double theta = Math.atan2(- fB, fA) / Astro.RAD;
	double dt = Astro.norm(theta - Math.atan2(fD, fE) / Astro.RAD, 360.0);
	if (dt >= 180.0)
	    dt -= 360.0;
	theta = Astro.norm(theta - dt / 2.0, 360.0);

	MapFunction mf = new MapFunction(r, theta, new Position(fC, fF));

	return mf;
    }

    // set R.A. and Decl. of all detected stars
    public void setAllDetectCoor ( MapFunction mf, Coor center_coor, double deg_per_pixel ) throws Exception {
	setAllDetectCoor(mf, center_coor, deg_per_pixel, new DistortionField());
    }
    public void setAllDetectCoor ( MapFunction mf, Coor center_coor, double deg_per_pixel, DistortionField df ) throws Exception {
	Vector list = getAllList();

	for (int i = 0 ; i < list.size() ; i++) {
	    Vector l = (Vector)list.elementAt(i);
	    for (int j = 0 ; j < l.size() ; j++) {
		PairData pair = (PairData)l.elementAt(j);
		DetectData sd_detect = (DetectData)pair.getFirstStarData();

		if (sd_detect != null) {
		    Position df_pos = df.getValue(sd_detect.getPosition());
		    Position pos = new Position(sd_detect.getX() + df_pos.getX(), sd_detect.getY() + df_pos.getY());
		    pos = mf.map(pos);
		    pos.rescale(deg_per_pixel);
		    CoorMapping coor = new CoorMapping(center_coor, pos);
		    sd_detect.setCoor(coor);

		    DetectData sd = (DetectData)sd_detect.getFirstAppendix();
		    while (sd != null) {
			df_pos = df.getValue(sd.getPosition());
			pos = new Position(sd.getX() + df_pos.getX(), sd.getY() + df_pos.getY());
			pos = mf.map(pos);
			pos.rescale(deg_per_pixel);
			coor = new CoorMapping(center_coor, pos);
			sd.setCoor(coor);
			sd = (DetectData)sd_detect.getNextAppendix();
		    }
		}
	    }
	}
    }

    // calculate errors of R.A., Decl.
    public Coor calculateErrorPosition ( ) throws Exception {
	double e_ra = 0.0;
	double e_decl = 0.0;
	for (int i = 0 ; i < getPairCount() ; i++) {
	    PairData pair = elementPairAt(i);
	    StarData sd_detect = pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    // O-C of a detected star (measured position - position in catalog)
	    Coor err = sd_detect.getCoor().residual(sd_catalog.getCoor());
	    err = new Coor(err.getRA() * 3600.0, err.getDecl() * 3600.0);

	    e_ra += err.getRA() * err.getRA();
	    e_decl += err.getDecl() * err.getDecl();
	}
	e_ra = Math.sqrt(e_ra / (double)getPairCount());
	e_decl = Math.sqrt(e_decl / (double)getPairCount());
	Coor error_pos = new Coor(e_ra, e_decl);
	return error_pos;
    }

    // calculate errors of (x,y)
    public Position calculateErrorPosition ( MapFunction mf ) throws Exception {
	Position err = new Position(0,0);
	Position err2 = new Position(0,0);
	for (int i = 0 ; i < getPairCount() ; i++) {
	    PairData pair = elementPairAt(i);
	    StarData sd_detect = pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    Position pos = mf.map(sd_detect.getPosition());

	    // O-C of a detected star (measured position - position in catalog)
	    err.add(new Position(pos.getX() - sd_catalog.getX(), pos.getY() - sd_catalog.getY()));
	    err2.add(new Position((pos.getX() - sd_catalog.getX()) * (pos.getX() - sd_catalog.getX()), (pos.getY() - sd_catalog.getY()) * (pos.getY() - sd_catalog.getY())));
	}
	err.rescale(1.0 / (double)getPairCount());
	err2.rescale(1.0 / (double)getPairCount());

	double dx = Math.sqrt(err2.getX() - err.getX() * err.getX());
	double dy = Math.sqrt(err2.getY() - err.getY() * err.getY());
	return new Position(dx, dy);
    }

    // calculate limiting magnitude
    public double calculateLimitingMag ( double faintest_mag ) throws Exception {
	int count = 0;
	double amt = 0.0;
	double amt2 = 0.0;
	double Amt = 0.0;
	double Amt2 = 0.0;
	for (int i = 0 ; i < getPairCount()  ; i++) {
	    PairData pair = elementPairAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    double dmag = sd_detect.getMag() - sd_catalog.getMag();
	    Amt += dmag;
	    Amt2 += dmag * dmag;

	    if (sd_detect.getMag() >= faintest_mag - 2.0) {
		amt += dmag;
		amt2 += dmag * dmag;
		count++;
	    }
	}
	if (count < 20) {
	    Amt /= (double)getPairCount();
	    Amt2 /= (double)getPairCount();
	    double sigma = Math.sqrt(Amt2 - Amt * Amt);
	    return faintest_mag - 3.0 * sigma;
	}

	amt /= (double)count;
	amt2 /= (double)count;
	double sigma = Math.sqrt(amt2 - amt * amt);
	return faintest_mag - 3.0 * sigma;
    }

    // adjust systematic shift of magnitude
    public MagnitudeAdjustment adjustMagnitude ( ) throws Exception {
	sortPairListInSecondMag();

	MagnitudeAdjustment ma = new MagnitudeAdjustment(PS_list);

	Vector l = getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);
	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		DetectData sd_detect = (DetectData)pair.getFirstStarData();
		StarData sd_catalog = pair.getSecondStarData();

		if (sd_detect != null) {
		    double m = sd_detect.getMag();
		    double dmag = ma.getDeltaMag(m);
		    sd_detect.setMag(sd_detect.getMag() + dmag);
		}
	    }
	}

	return ma;
    }

    // remove non-pair detect data out of the catalog area, or at edge
    public int removeDetectNonpairDataOutOfImage ( MapFunction mf, DistortionField df, Size area_size, double dxy ) throws Exception {
	int remove_count = 0;

	for (int i = 0 ; i < NP_list1.size() ; i++) {
	    PairData pair = (PairData)NP_list1.elementAt(i);
	    StarData sd = pair.getFirstStarData();
	    Position p = mf.map(sd.getPosition());
	    Position df_pos = df.getValue(p);
	    Position pos = new Position(p.getX() - df_pos.getX(), p.getY() - df_pos.getY());
	    if (pos.getX() < - (double)area_size.getWidth() / 2.0 + dxy  ||  pos.getX() > (double)area_size.getWidth() / 2.0 - dxy  ||  pos.getY() < - (double)area_size.getHeight() / 2.0 + dxy  ||  pos.getY() > (double)area_size.getHeight() / 2.0 - dxy) {
		NP_list1.removeElementAt(i);
		i--;
		remove_count++;
	    }
	}

	return remove_count;
    }

    // remove non-pair catalog data out of the image, or at edge
    public int removeCatalogNonpairDataOutOfImage ( MapFunction mf, DistortionField df, Size area_size, double dxy ) throws Exception {
	int remove_count = 0;

	MapFunction inv_mf = mf.inverse();
	for (int i = 0 ; i < NP_list2.size() ; i++) {
	    PairData pair = (PairData)NP_list2.elementAt(i);
	    StarData sd = pair.getSecondStarData();
	    Position p = inv_mf.map(sd.getPosition());
	    Position df_pos = df.getValue(p);
	    Position pos = new Position(p.getX() - df_pos.getX(), p.getY() - df_pos.getY());
	    if (pos.getX() < - (double)area_size.getWidth() / 2.0 + dxy  ||  pos.getX() > (double)area_size.getWidth() / 2.0 - dxy  ||  pos.getY() < - (double)area_size.getHeight() / 2.0 + dxy  ||  pos.getY() > (double)area_size.getHeight() / 2.0 - dxy) {
		NP_list2.removeElementAt(i);
		i--;
		remove_count++;
	    }
	}

	return remove_count;
    }

    // remove cosmic ray (only non-pair ones)
    public int removeCosmicRay ( ) throws Exception {
	int remove_count = 0;

	for (int i = 0 ; i < NP_list1.size() ; i++) {
	    PairData pair = (PairData)NP_list1.elementAt(i);
	    DetectData sd = (DetectData)pair.getFirstStarData();
	    if (sd.isCosmicRay()) {
		NP_list1.removeElementAt(i);
		i--;
		remove_count++;
	    }
	}

	return remove_count;
    }

    // process on data only in catalog which are too close to another data and blended into one star in the image
    public int blendCatalogStars ( MapFunction mf, DistortionField df ) throws Exception {
	MapFunction inv_mf = mf.inverse();

	int blended_count = 0;

	for (int i = 0 ; i < NP_list2.size() ; i++) {
	    PairData pair = (PairData)NP_list2.elementAt(i);
	    StarData sd_catalog = pair.getSecondStarData();
	    Position pos = inv_mf.map(sd_catalog.getPosition());

	    for (int j = 0 ; j < getPairCount() ; j++) {
		PairData pair2 = (PairData)PS_list.elementAt(j);
		DetectData sd_detect = (DetectData)pair2.getFirstStarData();
		StarData sdc = pair2.getSecondStarData();
		Position df_pos = df.getValue(sd_detect.getPosition());
		Position pos_detect = new Position(sd_detect.getX() + df_pos.getX(), sd_detect.getY() + df_pos.getY());
		double d_xy = (pos.getX() - pos_detect.getX()) * (pos.getX() - pos_detect.getX()) + (pos.getY() - pos_detect.getY()) * (pos.getY() - pos_detect.getY());
		if (d_xy <= sd_detect.getRadius() * sd_detect.getRadius() * 1.4) {	// 1.4 is ad hoc
		    sdc.append(sd_catalog);
		    NP_list2.removeElementAt(i);
		    i--;

		    // blended magnitude
		    sdc.setMag(Astro.blendMagnitude(sdc.getMag(), sd_catalog.getMag()));

		    blended_count++;

		    break;
		}
	    }
	}

	return blended_count;
    }

    // remove nonsense blending of catalog data
    public int removeNonsenseBlendingCatalogData ( double constant_dmag ) throws Exception {
	int remove_count = 0;

	double ap_mag = - Math.log(Math.pow(Astro.MAG_STEP, constant_dmag / 2.0) - 1.0) / Math.log(Astro.MAG_STEP);

	Vector list = getAllList();
	for (int i = 0 ; i < list.size() ; i++) {
	    Vector l = (Vector)list.elementAt(i);
	    for (int j = 0 ; j < l.size() ; j++) {
		PairData pair = (PairData)l.elementAt(j);
		StarData sd_detect = pair.getFirstStarData();
		StarData sd_catalog = pair.getSecondStarData();

		if (sd_catalog != null) {
		    Vector remove_list = new Vector();
		    StarData sd = sd_catalog.getFirstAppendix();
		    while (sd != null) {
			double dmag = sd.getMag() - sd_catalog.getMag();
			if (dmag >= 1.0  &&  dmag >= ap_mag)
			    remove_list.addElement(sd);
			sd = sd_catalog.getNextAppendix();
		    }

		    for (int k = 0 ; k < remove_list.size() ; k++) {
			sd = (StarData)remove_list.elementAt(k);
			sd_catalog.removeAppendix(sd);
			remove_count++;
		    }
		}
	    }
	}

	return remove_count;
    }

    // lists of data whose position or magnitude differs
    public int selectRemarkableList ( Coor error_pos, MagError error_mag ) throws Exception {
	int remarkable_count = 0;

	for (int i = 0 ; i < getPairCount() ; i++) {
	    PairData pair = (PairData)PS_list.elementAt(i);
	    StarData sd_detect = pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    Coor err = sd_detect.getCoor().residual(sd_catalog.getCoor());
	    err = new Coor(err.getRA() * 3600.0, err.getDecl() * 3600.0);

	    double m = sd_detect.getMag();
	    if (m > sd_catalog.getMag())
		m = sd_catalog.getMag();

	    if (Math.abs(sd_detect.getMag() - sd_catalog.getMag()) > error_mag.getRemarkableThreshold(m)) {
		pair.setType(PairType.RM);
		RM_list.addElement(pair);
		PS_list.removeElementAt(i);
		i--;
		remarkable_count++;
	    } else if ((Math.abs(err.getRA()) > 3.0 * error_pos.getRA()  ||
			Math.abs(err.getDecl()) > 3.0 * error_pos.getDecl())  ||
		       sd_detect.getFirstAppendix() != null) {
		pair.setType(PairType.RP);
		RP_list.addElement(pair);
		PS_list.removeElementAt(i);
		i--;
		remarkable_count++;
	    }
	}

	return remarkable_count;
    }

    // lists of data whose position or magnitude differs
    public int selectRemarkableList ( Position error_pos, MagError error_mag, MapFunction mf ) throws Exception {
	int remarkable_count = 0;

	for (int i = 0 ; i < getPairCount() ; i++) {
	    PairData pair = (PairData)PS_list.elementAt(i);
	    StarData sd_detect = pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    Position pos = mf.map(sd_detect.getPosition());

	    // O-C of a detected star (measured position - position in catalog)
	    Position err = new Position(pos.getX() - sd_catalog.getX(), pos.getY() - sd_catalog.getY());

	    double m = sd_detect.getMag();
	    if (m > sd_catalog.getMag())
		m = sd_catalog.getMag();

	    if (Math.abs(sd_detect.getMag() - sd_catalog.getMag()) > error_mag.getRemarkableThreshold(m)) {
		pair.setType(PairType.RM);
		RM_list.addElement(pair);
		PS_list.removeElementAt(i);
		i--;
		remarkable_count++;
	    } else if ((Math.abs(err.getX()) > 3.0 * error_pos.getX()  ||
			Math.abs(err.getY()) > 3.0 * error_pos.getY())  ||
		       sd_detect.getFirstAppendix() != null) {
		pair.setType(PairType.RP);
		RP_list.addElement(pair);
		PS_list.removeElementAt(i);
		i--;
		remarkable_count++;
	    }
	}

	return remarkable_count;
    }

    // remove mis-pair RP data such as:
    // data are faint, or position error is larger than 1.5 pixel
    public int removeMispairData ( double limit, double pixel_to_deg ) throws Exception {
	int remove_count = 0;

	for (int i = 0 ; i < RP_list.size() ; i++) {
	    PairData pair = (PairData)RP_list.elementAt(i);
	    StarData sd_detect = pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    if (sd_detect.getMag() > limit  &&  sd_catalog.getMag() > limit) {
		RP_list.removeElementAt(i);
		i--;
		remove_count++;
	    } else {
		Coor err = sd_detect.getCoor().residual(sd_catalog.getCoor());
		err = new Coor(err.getRA() * 3600.0, err.getDecl() * 3600.0);
		double d_pos = err.getRA();
		if (err.getDecl() > d_pos)
		    d_pos = err.getDecl();
		if (d_pos / 3600.0 / pixel_to_deg >= 1.5) {
		    RP_list.removeElementAt(i);
		    i--;
		    remove_count++;
		}
	    }
	}

	return remove_count;
    }

    // process in case of one star in catalog is actually close binary stars
    public int pairingBinaryStars ( Coor error_pos, double d_mag ) throws Exception {
	int paired_count = 0;

	for (int k = 0 ; k < 2 ; k++) {
	    Vector list = RP_list;
	    if (k == 1)
		list = RM_list;

	    for (int i = 0 ; i < list.size() ; i++) {
		PixyPairData pair = (PixyPairData)list.elementAt(i);
		StarData sd_detect = pair.getFirstStarData();
		StarData sd_catalog = pair.getSecondStarData();

		Coor err0 = sd_detect.getCoor().residual(sd_catalog.getCoor());
		err0 = new Coor(err0.getRA() * 3600.0, err0.getDecl() * 3600.0);

		double d_pos = (err0.getRA() * err0.getRA() + err0.getDecl() * err0.getDecl()) * 4.0;
		if (d_pos < (error_pos.getRA() * error_pos.getRA() + error_pos.getDecl() * error_pos.getDecl()) * 4.0)
		    d_pos = (error_pos.getRA() * error_pos.getRA() + error_pos.getDecl() * error_pos.getDecl()) * 4.0;
		if (d_pos > 60.0 * 60.0)	// maximum is 1 arcmin
		    d_pos = 60.0 * 60.0;

		for (int j = 0 ; j < NP_list1.size() ; j++) {
		    PairData pair2 = (PairData)NP_list1.elementAt(j);
		    StarData sd = pair2.getFirstStarData();
		    Coor err = sd.getCoor().residual(sd_catalog.getCoor());
		    err = new Coor(err.getRA() * 3600.0, err.getDecl() * 3600.0);
		    if (err.getRA() * err.getRA() + err.getDecl() * err.getDecl() < d_pos  &&  Math.abs(sd.getMag() - sd_detect.getMag()) < d_mag) {
			sd_detect.append(sd);
			NP_list1.removeElementAt(j);
			j--;
			paired_count++;
		    }
		}
	    }
	}

	return paired_count;
    }

    // set errors of RM, RP objects in memo
    public void setRemarkableErrorInMemo ( Coor error_pos, double pixel_to_deg ) throws Exception {
	for (int i = 0 ; i < RP_list.size() ; i++) {
	    PairData pair = (PairData)RP_list.elementAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    Coor err = sd_detect.getCoor().residual(sd_catalog.getCoor());
	    err = new Coor(err.getRA() * 3600.0, err.getDecl() * 3600.0);

	    String d_ra = Astro.toString(err.getRA(), 10, 8).trim();
	    String dp_ra = Astro.toString(err.getRA() / 3600.0 / pixel_to_deg, 10, 8).trim();
	    if (err.getRA() >= 0.0) {
		d_ra = "+" + d_ra;
		dp_ra = "+" + dp_ra;
	    }
	    String d_decl = Astro.toString(err.getDecl(), 10, 8).trim();
	    String dp_decl = Astro.toString(err.getDecl() / 3600.0 / pixel_to_deg, 10, 8).trim();
	    if (err.getDecl() >= 0.0) {
		d_decl = "+" + d_decl;
		dp_decl = "+" + dp_decl;
	    }

	    pair.addMemo("O-C(R.A., Decl.)=" + d_ra + "\", " + d_decl + "\"  (" + dp_ra + ", " + dp_decl + " pixels)");
	}
	for (int i = 0 ; i < RM_list.size() ; i++) {
	    PairData pair = (PairData)RM_list.elementAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    Coor err = sd_detect.getCoor().residual(sd_catalog.getCoor());
	    err = new Coor(err.getRA() * 3600.0, err.getDecl() * 3600.0);

	    String d_mag = Astro.toString(sd_detect.getMag() - sd_catalog.getMag(), 10, 7).trim();
	    if (sd_detect.getMag() - sd_catalog.getMag() >= 0.0)
		d_mag = "+" + d_mag;

	    if (Math.abs(err.getRA()) > 3.0 * error_pos.getRA()  ||
		 Math.abs(err.getDecl()) > 3.0 * error_pos.getDecl()) {
		String d_ra = Astro.toString(err.getRA(), 10, 8).trim();
		String dp_ra = Astro.toString(err.getRA() / 3600.0 / pixel_to_deg, 10, 8).trim();
		if (err.getRA() >= 0.0) {
		    d_ra = "+" + d_ra;
		    dp_ra = "+" + dp_ra;
		}
		String d_decl = Astro.toString(err.getDecl(), 10, 8).trim();
		String dp_decl = Astro.toString(err.getDecl() / 3600.0 / pixel_to_deg, 10, 8).trim();
		if (err.getDecl() >= 0.0) {
		    d_decl = "+" + d_decl;
		    dp_decl = "+" + dp_decl;
		}

		pair.addMemo("O-C(Mag)=" + d_mag + " mag  O-C(R.A., Decl.)=" + d_ra + "\", " + d_decl + "\"  (" + dp_ra + ", " + dp_decl + " pixels)");
	    } else {
		pair.addMemo("O-C(Mag)=" + d_mag + " mag");
	    }
	}
    }

    // set errors of RM, RP objects in memo
    public void setRemarkableErrorInMemo ( Position error_pos, MapFunction mf ) throws Exception {
	for (int i = 0 ; i < RP_list.size() ; i++) {
	    PairData pair = (PairData)RP_list.elementAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    Position pos = mf.map(sd_detect.getPosition());

	    // O-C of a detected star (measured position - position in catalog)
	    Position err = new Position(pos.getX() - sd_catalog.getX(), pos.getY() - sd_catalog.getY());

	    String dp_x = Astro.toString(err.getX(), 10, 8).trim();
	    if (err.getX() >= 0.0)
		dp_x = "+" + dp_x;
	    String dp_y = Astro.toString(err.getY(), 10, 8).trim();
	    if (err.getY() >= 0.0)
		dp_y = "+" + dp_y;

	    pair.addMemo("(dx,dy)=(" + dp_x + ", " + dp_y + ") pixels");
	}
	for (int i = 0 ; i < RM_list.size() ; i++) {
	    PairData pair = (PairData)RM_list.elementAt(i);
	    DetectData sd_detect = (DetectData)pair.getFirstStarData();
	    StarData sd_catalog = pair.getSecondStarData();

	    Position pos = mf.map(sd_detect.getPosition());

	    // O-C of a detected star (measured position - position in catalog)
	    Position err = new Position(pos.getX() - sd_catalog.getX(), pos.getY() - sd_catalog.getY());

	    String d_mag = Astro.toString(sd_detect.getMag() - sd_catalog.getMag(), 10, 7).trim();
	    if (sd_detect.getMag() - sd_catalog.getMag() >= 0.0)
		d_mag = "+" + d_mag;

	    if (Math.abs(err.getX()) > 3.0 * error_pos.getX()  ||
		 Math.abs(err.getY()) > 3.0 * error_pos.getY()) {
		String dp_x = Astro.toString(err.getX(), 10, 8).trim();
		if (err.getX() >= 0.0)
		    dp_x = "+" + dp_x;
		String dp_y = Astro.toString(err.getY(), 10, 8).trim();
		if (err.getY() >= 0.0)
		    dp_y = "+" + dp_y;

		pair.addMemo("dMag=" + d_mag + " mag  (dx,dy)=(" + dp_x + ", " + dp_y + ") pixels");
	    } else {
		pair.addMemo("dMag=" + d_mag + " mag");
	    }
	}
    }

    // set (x,y) of NC objects
    public void createDetectDataOfNC ( MapFunction mf, DistortionField df, Size area_size ) throws Exception {
	MapFunction inv_mf = mf.inverse();
	for (int i = 0 ; i < NP_list2.size() ; i++) {
	    PixyPairData pair = (PixyPairData)NP_list2.elementAt(i);
	    StarData sd = pair.getSecondStarData();
	    Position p = inv_mf.map(sd.getPosition());
	    Position df_pos = df.getValue(p);
	    Position pos = new Position(p.getX() - df_pos.getX(), p.getY() - df_pos.getY());
	    pos.add(new Position((double)area_size.getWidth() / 2.0, (double)area_size.getHeight() / 2.0));
	    pair.setDetectPosition(pos);
	}
    }

    // set (x,y) of ND objects on the other image
    public void createDetectDataOfND ( MapFunction mf, DistortionField df, Size area_size ) throws Exception {
	for (int i = 0 ; i < NP_list1.size() ; i++) {
	    PixyPairData pair = (PixyPairData)NP_list1.elementAt(i);
	    StarData sd = pair.getFirstStarData();
	    Position p = mf.map(sd.getPosition());
	    Position df_pos = df.getValue(p);
	    Position pos = new Position(p.getX() - df_pos.getX(), p.getY() - df_pos.getY());
	    pos.add(new Position((double)area_size.getWidth() / 2.0, (double)area_size.getHeight() / 2.0));
	    pair.setCatalogPosition(pos);
	}
    }

    // set (x,y) of all catalog data
    public void setAllCatalogPosition ( Coor center_coor, double deg_per_pixel ) throws Exception {
	Vector list = getAllList();

	for (int i = 0 ; i < list.size() ; i++) {
	    Vector l = (Vector)list.elementAt(i);
	    for (int j = 0 ; j < l.size() ; j++) {
		PairData pair = (PairData)l.elementAt(j);
		StarData sd_catalog = pair.getSecondStarData();

		if (sd_catalog != null) {
		    Position pos = new CoorMapping(sd_catalog.getCoor()).mappingXY(center_coor);
		    pos.rescale(deg_per_pixel);
		    sd_catalog.setPosition(pos);

		    StarData sd = sd_catalog.getFirstAppendix();
		    while (sd != null) {
			pos = new CoorMapping(sd.getCoor()).mappingXY(center_coor);
			pos.rescale(deg_per_pixel);
			sd.setPosition(pos);
			sd = sd_catalog.getNextAppendix();
		    }
		}
	    }
	}
    }

    // set at_edge flag of detect data
    public void selectAtEdge ( Size size, Position base_pos ) throws Exception {
	Vector list = getAllList();

	for (int i = 0 ; i < list.size() ; i++) {
	    Vector l = (Vector)list.elementAt(i);
	    for (int j = 0 ; j < l.size() ; j++) {
		PairData pair = (PairData)l.elementAt(j);
		DetectData sd_detect = (DetectData)pair.getFirstStarData();

		if (sd_detect != null) {
		    Position pos = new Position(sd_detect.getX() + base_pos.getX(), sd_detect.getY() + base_pos.getY());
		    if (pos.getX() < 2  ||  pos.getY() < 2  ||  pos.getX() > size.getWidth() - 2  ||  pos.getY() > size.getHeight() - 2)
			sd_detect.setEdge(true);

		    DetectData sd = (DetectData)sd_detect.getFirstAppendix();
		    while (sd != null) {
			pos = new Position(sd.getX() + base_pos.getX(), sd.getY() + base_pos.getY());
			if (pos.getX() < 2  ||  pos.getY() < 2  ||  pos.getX() > size.getWidth() - 2  ||  pos.getY() > size.getHeight() - 2)
			    sd.setEdge(true);

			sd = (DetectData)sd_detect.getNextAppendix();
		    }
		}
	    }
	}
    }

    // remove illegal differential data based on the checking magnitude
    public void removeIllegalDifferentialPairs ( MagError error_mag, Size size1, Size size2 ) throws Exception {
	Vector l = getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);
	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);

		if (pair.getType() != PairType.PS) {
		    DetectData sd1 = (DetectData)pair.getFirstStarData();
		    DetectData sd2 = (DetectData)pair.getSecondStarData();

		    if (sd1 != null  &&  sd2 != null) {
			if (5 <= sd1.getX()  &&  sd1.getX() <= size1.getWidth() - 5  &&
			    5 <= sd1.getY()  &&  sd1.getY() <= size1.getHeight() - 5  &&
			    5 <= sd2.getX()  &&  sd2.getX() <= size2.getWidth() - 5  &&
			    5 <= sd2.getY()  &&  sd2.getY() <= size2.getHeight() - 5) {
			    if (sd1.getCheckMag() < Astro.ILLEGAL_MAG - 1.0  ||  sd2.getCheckMag() < Astro.ILLEGAL_MAG - 1.0) {
				double m = sd1.getCheckMag();
				if (m > sd2.getCheckMag())
				    m = sd2.getCheckMag();
			 	if (pair.getType() != PairType.RP) {
				    if (Math.abs(sd1.getCheckMag() - sd2.getCheckMag()) < error_mag.getRemarkableThreshold(m))
					pair.setIllegal();
				}
			    } else {
				pair.setIllegal();
			    }
			} else {
			    pair.setIllegal();
			}
		    }
		}
	    }
	}
    }

    ////////////////////////////////////////////////////////////////

    // color of stars
    public void setStarColor ( ) throws Exception {
	Vector l = getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);
	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		StarData sd_detect = pair.getFirstStarData();
		StarData sd_catalog = pair.getSecondStarData();

		if (sd_detect != null)
		    sd_detect.setColorAll(pair.type.getColor());
		if (sd_catalog != null)
		    sd_catalog.setColorAll(pair.type.getColor());
	    }
	}
    }

    // shape of stars
    public void setStarPlotAsCircle ( ) throws Exception {
	Vector l = getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);
	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		StarData sd_detect = pair.getFirstStarData();
		StarData sd_catalog = pair.getSecondStarData();

		if (pair.type != PairType.PS) {
		    if (sd_detect != null)
			sd_detect.setPlotCircleAll();
		    if (sd_catalog != null)
			sd_catalog.setPlotCircleAll();
		}
	    }
	}
    }
}
