/*
 * SelectStarListIndex.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.Point;
import java.util.Vector;
import misao.util.*;

public class SelectStarListIndex extends StarListIndex {
    public SelectStarListIndex ( StarList list, Position pos1, Position pos2, int mesh, int star_count ) {
	StarList l = new StarList();
	int count = 0;
	for (int i = 0 ; count < star_count  &&  i < list.size() ; i++) {
	    StarData sd = list.elementAt(i);
	    if (sd instanceof DetectData) {
		// do not use detected stars at edge or cosmic rays
		DetectData dsd = (DetectData)sd;
		if (! (dsd.isEdge()  ||  (dsd.isCosmicRay()  &&  dsd.getArea() < 3))) {
		     l.addElement(sd);
		     count++;
		 }
	    } else {
		l.addElement(sd);
		count++;
	    }
	}

	double w = pos2.getX() - pos1.getX();
	double h = pos2.getY() - pos1.getY();
	mesh_unit = (w > h ? w : h) / (double)mesh;
	positionA = new Position(pos1);
	positionB = new Position(pos2);
	int width = (int)((pos2.getX() - pos1.getX()) / mesh_unit) + 1;
	int height = (int)((pos2.getY() - pos1.getY()) / mesh_unit) + 1;
	if (width * mesh_unit - (pos2.getX() - pos1.getX()) > mesh_unit / 2.0)
	    width--;
	if (height * mesh_unit - (pos2.getY() - pos1.getY()) > mesh_unit / 2.0)
	    height--;
	mesh_size = new Size(width, height);
	table = new StarList[mesh_size.getWidth()][mesh_size.getHeight()];

	pt_list = new Vector();
	pt_index = 0;

	for (int i = 0 ; i < mesh_size.getWidth() ; i++) {
	    for (int j = 0 ; j < mesh_size.getHeight() ; j++) {
		table[i][j] = new StarList();
	    }
	}

	for (int i = 0 ; i < l.size() ; i++) {
	    StarData sd = l.elementAt(i);
	    Point pt = findIndex(sd, new MapFunction());
	    table[pt.x][pt.y].addElement(sd);
	}
    }

    public StarList select ( int count_per_mesh, int count_total ) {
	StarList[] list = selectTwoLists(count_per_mesh, count_total);
	StarList l = new StarList(list[0]);
	l.add(list[1]);
	return l;
    }

    public StarList[] selectTwoLists ( int count_per_mesh, int count_total ) {
	StarList[] list = new StarList[2];
	list[0] = new StarList();
	list[1] = new StarList();

	StarList l2 = new StarList();
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < mesh_size.getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < mesh_size.getWidth() ; pt.x++) {
		if (table[pt.x][pt.y].size() > 0) {
		    StarList l = new StarList(table[pt.x][pt.y]);
		    l.sortMagAscendant();
		    int count = 0;
		    while (l.size() > 0) {
			if (count < count_per_mesh) {
			    list[0].addElement(l.elementAt(0));
			    count++;
			} else {
			    l2.addElement(l.elementAt(0));
			}
			l.removeElementAt(0);
		    }
		}
	    }
	}

	l2.sortMagAscendant();
	for (int i = 0 ; i < l2.size()  &&  i < count_total ; i++) {
	    list[1].addElement(l2.elementAt(i));
	}

	return list;
    }
}
