/*
 * TriStarsIndex.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.Point;
import java.util.Vector;
import misao.util.*;

public class TriStarsIndex {
    private static final int mesh_size = 40;
    private static final double mesh_unit = 1.1;

    private Vector table[][];

    public TriStarsIndex ( ) {
	table = new Vector[mesh_size][mesh_size];

	for (int i = 0 ; i < mesh_size ; i++) {
	    for (int j = 0 ; j < mesh_size ; j++) {
		table[i][j] = new Vector();
	    }
	}
    }

    public TriStarsIndex ( StarList list ) {
	this();

	for (int i = 0 ; i < list.size() - 2 ; i++) {
	    for (int j = i + 1 ; j < list.size() - 1 ; j++) {
		for (int k = j + 1 ; k < list.size() ; k++) {
		    TriStars tri = new TriStars(list.elementAt(i), list.elementAt(j), list.elementAt(k));
		    addTriStars(tri);
		}
	    }
	}
    }

    public void addTriStars ( TriStars tri ) {
	Point pt = findIndex(tri);
	if (pt != null)
	    table[pt.x][pt.y].addElement(tri);
    }

    public Point findIndex ( TriStars tri ) {
	TriStarsForm form = tri.getForm();
	if (form == null)
	    return null;

	int i = (int)(Math.log(form.getRatio1()) / Math.log(mesh_unit));
	int j = (int)(Math.log(form.getRatio2()) / Math.log(mesh_unit));
	if (i >= mesh_size)
	    return null;
	if (j >= mesh_size)
	    return null;
	return new Point(i, j);
    }

    public Vector findList ( TriStars tri ) {
	Point pt = findIndex(tri);
	if (pt == null)
	    return new Vector();

	Vector list = new Vector();
	Point dpt = new Point(0, 0);
	for (dpt.y = -1 ; dpt.y <= 1 ; dpt.y++) {
	    for (dpt.x = -1 ; dpt.x <= 1 ; dpt.x++) {
		if (0 <= pt.x + dpt.x  &&  pt.x + dpt.x < mesh_size  &&  0 <= pt.y + dpt.y  &&  pt.y + dpt.y < mesh_size) {
		    Vector l = table[pt.x + dpt.x][pt.y + dpt.y];
		    for (int i = 0 ; i < l.size() ; i++)
			list.addElement(l.elementAt(i));
		}
	    }
	}

	return list;
    }

    // the order is best for matching
    Vector getAllList ( ) {
	Vector l = new Vector();

	for (int j = 0 ; j < mesh_size ; j++) {
	    for (int i = mesh_size - 1 ; i >= 0 ; i--) {
		for (int k = 0 ; k < table[i][j].size() ; k++)
		    l.addElement(table[i][j].elementAt(k));
	    }
	}

	return l;
    }
}
