/*
 * GcvsPatch.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
ANDBO P 2000 225838.16 +453151.4 GSC
ANDEF P 2000 003943.92 +463320.4 GSC
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class GcvsPatch extends ReferenceCatalog {
    public GcvsPatch ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	StringTokenizer st = new StringTokenizer(record);
	if (st.countTokens() < 5)
	    return null;

	String dbvs_name = st.nextToken();
	st.nextToken();
	String precession = st.nextToken();
	String RA = st.nextToken();
	String Decl = st.nextToken();

	int p = RA.indexOf('.');
	if (p < 0)
	    p = RA.length();
	while (p < 6) {
	    RA = "0" + RA;
	    p++;
	}
	p = Decl.indexOf('.');
	if (p < 0)
	    p = Decl.length();
	while (p < 7) {
	    Decl = Decl.charAt(0) + "0" + Decl.substring(1);
	    p++;
	}

	String name = dbvs_name;
	String cons = dbvs_name.substring(0,3);
	if (ConstellationTable.isConstellationCode(cons))
	    name = dbvs_name.substring(3) + " " + dbvs_name.substring(0,3);
	dbvs_name = VariableStarName.toFile(dbvs_name);

	CoorMapping coor = new CoorMapping(Integer.parseInt(RA.substring(0,2)), Integer.parseInt(RA.substring(2,4)), (double)(Float.valueOf(RA.substring(4)).floatValue()), (Decl.charAt(0) == '-'), Integer.parseInt(Decl.substring(1,3)), Integer.parseInt(Decl.substring(3,5)), (double)(Float.valueOf(Decl.substring(5)).floatValue()));
	if (precession.equals("1950"))
	    coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	record = name + "  R.A.=" + coor.toStringRA() + "  Decl.=" + coor.toStringDecl();

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, record);

	return data;
    }

    public static ReferenceData updateData ( ReferenceData orig_data, ReferenceData patch_data ) {
	String record = orig_data.getRecord();
	int p = record.indexOf("Decl.");
	p = record.indexOf("  ", p);
	if (p < 0)
	    p = record.indexOf("Decl.") + 17;
	record = patch_data.getRecord() + record.substring(p);
	ReferenceData data = new ReferenceData(orig_data.getName(), orig_data.getDbvsName(), patch_data.getCoor(), record, orig_data.getPositionError(), orig_data.getType());
	return data;
    }
}
