/*
 * GcvsPatchWilliams.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
0010    Object        R.A. (J2000.0) Decl.   Id.                                Chart (pref.)/                 Pos. Ref             Note
0011                                         PPM     GSC         USNO A         (Rough) Pos.
010012  RT     And  23 11 10.05 +53 01 33.3 *041561  3998.02167  1425.14866861  BD                             Unpub.
010021  SU     And  00 04 36.41 +43 33 05.0 *042538  2793.00243  1275.00049189  BD                             Unpub.
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class GcvsPatchWilliams extends ReferenceCatalog {
    public GcvsPatchWilliams ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.length() < 111)
	    return null;

	if (('0' <= record.charAt(0)  &&  record.charAt(0) <= '9')  &&
	    ('0' <= record.charAt(1)  &&  record.charAt(1) <= '9')  &&
	    ('0' <= record.charAt(2)  &&  record.charAt(2) <= '9')  &&
	    ('0' <= record.charAt(3)  &&  record.charAt(3) <= '9')  &&
	    ('0' <= record.charAt(4)  &&  record.charAt(4) <= '9')  &&
	    ('0' <= record.charAt(5)  &&  record.charAt(5) <= '9')  &&
	    ('0' <= record.charAt(20)  &&  record.charAt(20) <= '9')  &&
	    ('0' <= record.charAt(21)  &&  record.charAt(21) <= '9')  &&
	    ('0' <= record.charAt(23)  &&  record.charAt(23) <= '9')  &&
	    ('0' <= record.charAt(24)  &&  record.charAt(24) <= '9')  &&
	    ('0' <= record.charAt(26)  &&  record.charAt(26) <= '9')  &&
	    ('0' <= record.charAt(27)  &&  record.charAt(27) <= '9')  &&
	    ('0' <= record.charAt(33)  &&  record.charAt(33) <= '9')  &&
	    ('0' <= record.charAt(34)  &&  record.charAt(34) <= '9')  &&
	    ('0' <= record.charAt(36)  &&  record.charAt(36) <= '9')  &&
	    ('0' <= record.charAt(37)  &&  record.charAt(37) <= '9')  &&
	    ('0' <= record.charAt(39)  &&  record.charAt(39) <= '9')  &&
	    ('0' <= record.charAt(40)  &&  record.charAt(40) <= '9')  &&
	    ('0' <= record.charAt(42)  &&  record.charAt(42) <= '9')) {

	    String cons = record.substring(15,18);
	    String pre_name = record.substring(8,18);
	    String RA = record.substring(20,31);
	    String Decl = record.substring(32,43);
	    String GSC = record.substring(53,63).trim();
	    String refer = record.substring(80,111).trim();

	    if (GSC.length() > 0)
		GSC = "GSC" + GSC;
	    String USNO = record.substring(65,78).trim();
	    if (USNO.length() > 0)
		USNO = "USNO" + USNO;

	    String name = pre_name.substring(0,1);
	    String dbvs_name = pre_name.substring(0,1);
	    for (int i = 1 ; i < 10 ; i++) {
		if (pre_name.charAt(i) != ' '  ||  pre_name.charAt(i-1) != ' ')
		    name += pre_name.substring(i,i+1);
		if (pre_name.charAt(i) != ' ')
		    dbvs_name += pre_name.substring(i,i+1);
	    }
	    name = name.trim();
	    dbvs_name = dbvs_name.toUpperCase().trim();

	    if (ConstellationTable.isConstellationCode(cons)) {
		int p = dbvs_name.length();
		name = name.toUpperCase();
		dbvs_name = dbvs_name.substring(p-3) + dbvs_name.substring(0,p-3);
	    } else {
		name = dbvs_name;
	    }

	    CoorMapping coor = new CoorMapping(Coor.createCoor(RA + " " + Decl));

	    record = name + "  R.A.=" + RA + "  Decl.=" + Decl;
	    if (GSC.length() > 0)
		record += "  " + GSC;
	    if (USNO.length() > 0)
		record += "  " + USNO;
	    record += "  " + refer;

	    ReferenceData data = new ReferenceData(name, dbvs_name, coor, record);

	    return data;
	}

	return null;
    }

    public static ReferenceData updateData ( ReferenceData orig_data, ReferenceData patch_data ) {
	String orig_record = orig_data.getRecord();
	int p = orig_record.indexOf("Decl.");
	p = orig_record.indexOf("  ", p);
	if (p < 0)
	    p = orig_record.indexOf("Decl.") + 17;

	String patch_record = patch_data.getRecord();
	int q = patch_record.indexOf("Decl.");
	q = patch_record.indexOf("  ", p);
	if (q < 0)
	    q = patch_record.indexOf("Decl.") + 17;

	String record = patch_record.substring(0,q) + orig_record.substring(p);
	ReferenceData data = new ReferenceData(orig_data.getName(), orig_data.getDbvsName(), patch_data.getCoor(), record, orig_data.getPositionError(), orig_data.getType());
	return data;
    }
}
