/*
 * IrasPsc.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
00000-0359  0 0 10- 35920 47  8 67 22.500E-013.291E-011.209E+002.490E+001133 0    0  0 10 11    0    0  108  115 HAA-1000001000000023 20 11 21000100010001000
00000+6549  0 0 19+654915 15  4 39 41.901E+004.447E-017.934E+002.583E+013311 0    4  7  0  0  210   83    0    0AB   1003203600100C97121 54 41000100010001000
 *
 */

package misao.ref.catalog;

import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class IrasPsc extends ReferenceCatalog {
    public IrasPsc ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
//	if (record.charAt(72) == ' '  ||  record.charAt(72) == '1')
//	    return null;

	String var = record.substring(116,118).trim();
//	if (var.length() == 0  ||  var.equals("-1"))
//	    return null;

	String name = "IRAS " + record.substring(0, 11).trim();
	String dbvs_name = "IRAS" + record.substring(0, 11).trim();

	CoorMapping coor = new CoorMapping(Integer.parseInt(record.substring(11,13).trim()), Integer.parseInt(record.substring(13,15).trim()), (double)Integer.parseInt(record.substring(15,18).trim()) / 10.0, (record.charAt(18) == '-'), Integer.parseInt(record.substring(19,21).trim()), Integer.parseInt(record.substring(21,23).trim()), (double)Integer.parseInt(record.substring(23,25).trim()));
	coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	String s = name + "  R.A.=" + coor.toStringRA() + "  Decl.=" + coor.toStringDecl() + "  (" + record.substring(25,28).trim() + " x " + record.substring(28,31).trim() + "\", p.a.=" + record.substring(31,34).trim() + ")";

	String flux = record.substring(36,45).trim();
	if (flux.length() > 0) {
	    s += "  flux(12)=";
	    if (record.charAt(72) == '1')
		s += '<';
	    s += Astro.toString(Astro.doubleValueOf(flux), 5, 2).trim();
	}
	flux = record.substring(45,54).trim();
	if (flux.length() > 0) {
	    s += "  flux(25)=";
	    if (record.charAt(73) == '1')
		s += '<';
	    s += Astro.toString(Astro.doubleValueOf(flux), 5, 2).trim();
	}
	flux = record.substring(54,63).trim();
	if (flux.length() > 0) {
	    s += "  flux(60)=";
	    if (record.charAt(74) == '1')
		s += '<';
	    s += Astro.toString(Astro.doubleValueOf(flux), 5, 2).trim();
	}
	flux = record.substring(63,72).trim();
	if (flux.length() > 0) {
	    s += "  flux(100)=";
	    if (record.charAt(75) == '1')
		s += '<';
	    s += Astro.toString(Astro.doubleValueOf(flux), 5, 2).trim();
	}

	if (var.length() > 0  &&  ! var.equals("-1"))
		s += "  " + var + "% variable";

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, s);

	return data;
    }
}
