/*
 * Mcg.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
MCG+15-01-001           0003.  +88 04 16.    2.   0.5  16.  13. 352
MCG+15-01-002           0455.  +88 19 15.    1.5  1.   10.   4. 252
 *
 */

package misao.ref.catalog;

import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class Mcg extends ReferenceCatalog {
    public Mcg ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	String name = record.substring(0,14).trim();
	String dbvs_name = name.toUpperCase();

	String alias = record.substring(15,24).trim();

	CoorMapping coor = null;
	int ra_type = 0;
	if (record.charAt(29) != ' ')
	    ra_type = 1;
	if (record.charAt(30) != ' ')
	    ra_type = 2;

	if (ra_type == 0) {
	    coor = new CoorMapping(Integer.parseInt(record.substring(24,26)), Integer.parseInt(record.substring(26,28)), 0.0 * 60.0, (record.charAt(31) == '-'), Integer.parseInt(record.substring(32,34)), Integer.parseInt(record.substring(35,37)), 0.0);
	} else if (ra_type == 1) {
	    coor = new CoorMapping(Integer.parseInt(record.substring(24,26)), Integer.parseInt(record.substring(26,28)), ((double)Integer.parseInt(record.substring(29,30))) / 10.0 * 60.0, (record.charAt(31) == '-'), Integer.parseInt(record.substring(32,34)), Integer.parseInt(record.substring(35,37)), 0.0);
	} else if (ra_type == 2) {
	    coor = new CoorMapping(Integer.parseInt(record.substring(24,26)), Integer.parseInt(record.substring(26,28)), ((double)Integer.parseInt(record.substring(29,31))) / 100.0 * 60.0, (record.charAt(31) == '-'), Integer.parseInt(record.substring(32,34)), Integer.parseInt(record.substring(35,37)), 0.0);
	}
	coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	String RA = "";
	if (ra_type == 0)
	    RA = Astro.toStringZeroPadding(coor.getRA_h(), 2) + " " + Astro.toStringZeroPadding((double)coor.getRA_m() + coor.getRA_s() / 60.0, 2, 2);
	else if (ra_type == 1)
	    RA = Astro.toStringZeroPadding(coor.getRA_h(), 2) + " " + Astro.toStringZeroPadding((double)coor.getRA_m() + coor.getRA_s() / 60.0, 4, 2);
	else if (ra_type == 2)
	    RA = Astro.toStringZeroPadding(coor.getRA_h(), 2) + " " + Astro.toStringZeroPadding((double)coor.getRA_m() + coor.getRA_s() / 60.0, 5, 2);
	String Decl = (coor.getDecl() < 0.0 ? '-' : '+') + Astro.toStringZeroPadding(coor.getAbsDecl_d(), 2) + " " + Astro.toStringZeroPadding((double)coor.getAbsDecl_m() + coor.getAbsDecl_s() / 60.0, 2, 2);

	String s = name + "  ";
	if (alias.length() > 0)
	    s += alias + "  ";
	s += "R.A.=" + RA + "  Decl.=" + Decl;
	if (record.length() >= 39) {
	    String mag = record.substring(38).trim();
	    if (mag.length() > 0) {
		if (mag.indexOf(' ') > 0)
		    mag = mag.substring(0, mag.indexOf(' '));
		if (mag.charAt(mag.length() - 1) == '.')
		    mag = mag.substring(0, mag.length() - 1);
		s += "  " + mag + " mag";
	    }
	}

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, s);

	return data;
    }
}
