/*
 * MisvV.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
Code     R.A. (J2000.0) Decl.  Max    Min   Type    Spectrum  Period      Epoch         ID
----------------------------------------------------------------------------------------------------
MisV0001 175226.59 -174000.7  13.9C  16.9C  M       ?           390:      2451280:      
MisV0002 072403.56 +412602.0  13.3C  14.7C  E:      ?              ?             ?      GSC2965.0210 USNO-A1.0_1275.06972791

Code     R.A. (J2000.0) Decl.  Max    Min   Type    Spectrum  ID               
------------------------------------------------------------------------------
MisV0001 175226.59 -174000.7  13.9C  16.9C  M       ?         
MisV0002 072403.56 +412602.0  13.3C  14.7C  E:      ?         GSC2965.0210 USNO-A1.0_1275.06972791
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class MisV extends ReferenceCatalog {
    public MisV ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.length() < 62  ||  ! record.substring(0,4).equals("MisV"))
	    return null;

	String name = record.substring(0,8);
	String ra = record.substring(9,18);
	String decl = record.substring(19,28);
	String max = record.substring(29,36).trim();
	String min = record.substring(36,43).trim();
	String type = record.substring(44,52).trim();
	String spectrum = record.substring(52,62).trim();
	String period = record.substring(62,74).trim();
	String epoch = record.substring(74,88).trim();
	String id = record.substring(88).trim();

	String dbvs_name = name.toUpperCase();

	CoorMapping coor = new CoorMapping(Integer.parseInt(ra.substring(0,2)), Integer.parseInt(ra.substring(2,4)), Double.valueOf(ra.substring(4,9)).doubleValue(), (decl.charAt(0) == '-'), Integer.parseInt(decl.substring(1,3)), Integer.parseInt(decl.substring(3,5)), Double.valueOf(decl.substring(5,9)).doubleValue());

	String s = name + "  R.A.=" + ra.substring(0,2) + "h" + ra.substring(2,4) + "m" + ra.substring(4,6) + "s" + ra.substring(6,9) + "  Decl.=" + decl.substring(0,3) + "o" + decl.substring(3,5) + "'" + decl.substring(5,7) + "\"" + decl.substring(7,9) + "  Mag.: " + max + " - " + min + "  Type: " + type + "  Sp: " + spectrum + "  Period: " + period + "  Epoch: " + epoch;
	if (id.length() > 0)
	    s += "  " + id;

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, s);

	return data;
    }
}
