/*
 * OgleBul.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
<TABLE>
<TR>
<TH><TH>Event<TH>Field<TH>Star No<TH>RA (J2000)<TH>Dec (J2000) 
<TH>T<SUB>max</SUB> (HJD) <TH>T<SUB>max</SUB> (UT) <TH>tau
<TH>A<SUB>max</SUB> <TH>D<SUB>mag</SUB> <TH> I<SUB>0</SUB>
</TR>

<TR>
<TD>
<TD><A HREF="bul-01.html">1999-BUL-01</A>
<TD>BUL_SC39  <TD ALIGN="RIGHT"> 623258
<TD>17:56:04.05
<TD>-30:08:53.0
<TD>2451172.278
<TD>1998-12-24.78
<TD>88.864
<TD>1.472
<TD>0.420
<TD>16.968
</TR>
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class OgleBul extends ReferenceCatalog {
    private boolean in_table;
    private boolean skip_header;
    private int line;

    private String event;
    private String ra;
    private String decl;
    private String Tmax;
    private String tau;
    private String Amax;
    private String Dmag;
    private String I0;

    public OgleBul ( ) {
	in_table = false;
	skip_header = false;
	line = 0;
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.length() < 4)
	    return null;

	if (record.length() >= 7  &&  record.substring(0,7).equals("<TABLE>")) {
	    in_table = true;
	    return null;
	}
	if (record.length() >= 8  &&  record.substring(0,8).equals("</TABLE>")) {
	    in_table = false;
	    return null;
	}

	if (in_table) {
	    if (record.length() >= 5  &&  record.substring(0,5).equals("</TR>")) {
		if (skip_header) {
		    String name = "OGLE-" + event;
		    String dbvs_name = name;

		    CoorMapping coor = new CoorMapping(Coor.createCoor(ra.replace(':',' ') + " " + decl.replace(':',' ')));

		    record = name + "  " + ra + "  " + decl + "  Tmax: " + Tmax + "  tau: " + tau + "  Amax: " + Amax + "  Dmag: " + Dmag + "  I0: " + I0;

		    ReferenceData data = new ReferenceData(name, dbvs_name, coor, record);

		    line = 0;
		    return data;
		} else {
		    skip_header = true;
		    return null;
		}
	    } else if (skip_header  &&  record.length() >= 4  &&  record.substring(0,4).equals("<TD>")) {
		line++;

		String s = record.substring(4);
		int p;
		switch (line) {
		    case 1:
			break;
		    case 2:
			p = s.indexOf('>');
			if (p < 0)
			    event = s;
			else {
			    s = s.substring(p+1);
			    p = s.indexOf('<');
			    if (p < 0)
				p = s.length();
			    event = s.substring(0,p);
			}
			break;
		    case 3:
			break;
		    case 4:
			ra = s;
			break;
		    case 5:
			decl = s;
			break;
		    case 6:
			break;
		    case 7:
			Tmax = s;
			break;
		    case 8:
			tau = s;
			break;
		    case 9:
			Amax = s;
			break;
		    case 10:
			Dmag = s;
			break;
		    case 11:
			I0 = s;
			break;
		}
	    }
	}

	return null;
    }
}
