/*
 * Tass.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
<TR>
<TD><A href="TASS_J004615_1-000219.htm">TASS J004615.1-000219</A></TD>
<TD> 11.5630</TD>
<TD>-0.0387</TD>
<TD>E</TD>
<TD>10.09</TD>
<TD>10.52</TD>
<TD>V</TD></TR>
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class Tass extends ReferenceCatalog {
    private int line;

    private String name;
    private String dbvs_name;
    private double ra;
    private double decl;
    private String type;
    private String max;
    private String min;
    private String filter;

    public Tass ( ) {
	line = -1;

	name = "";
	dbvs_name = "";
	ra = 0;
	decl = 0;
	type = "";
	max = "";
	min = "";
	filter = "";
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.indexOf("STRONG") >= 0)
	    return null;

	if (record.indexOf("<TR>") >= 0)
	    line = 0;

	int p = record.indexOf("<TD>");
	int q = record.indexOf("</TD>");

	if (p >= 0  &&  q > p) {
	    String s = record.substring(p+4, q);
	    if (s.indexOf("<A ") >= 0) {
		p = s.indexOf(">");
		q = s.indexOf("</A>");

		if (p >= 0  &&  q > p)
		    s = s.substring(p+1, q);
	    }

	    switch (line) {
		case 0:
		    name = s;
		    dbvs_name = s.replace(' ', '_').toUpperCase();
		    break;
		case 1:
		    ra = Astro.doubleValueOf(s);
		    break;
		case 2:
		    decl = Astro.doubleValueOf(s);
		    break;
		case 3:
		    type = s;
		    break;
		case 4:
		    max = s;
		    break;
		case 5:
		    min = s;
		    break;
		case 6:
		    filter = s;
		    break;
	    }

	    line++;
	}

	if (record.indexOf("</TR>") >= 0) {
	    Coor coor = new Coor(ra, decl);

	    record = name + "  ";
	    record += coor.toStringRA() + "  ";
	    record += coor.toStringDecl() + "  ";
	    if (type.length() > 0)
		record += type + "  ";
	    record += max + " - " + min + " " + filter;

	    ReferenceData data = new ReferenceData(name, dbvs_name, coor, record);

	    name = "";
	    dbvs_name = "";
	    ra = 0;
	    decl = 0;
	    type = "";
	    max = "";
	    min = "";
	    filter = "";

	    return data;
	}

	return null;
    }
}
