/*
 * Catalog.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class Catalog {
    protected static final int CATALOG_CDROM = 0;
    protected static final int CATALOG_WWW = 1;

    protected String code;
    protected String name;
    protected double mag_limit;
    protected double fov_limit;
    protected int type;
    protected boolean all_sky;
    protected boolean image;

    protected Catalog ( String _code, String _name, double _mag_limit, double _fov_limit, int _type, boolean _all_sky, boolean _image ) {
	code = _code;
	name = _name;
	mag_limit = _mag_limit;
	fov_limit = _fov_limit;
	type = _type;
	all_sky = _all_sky;
	image = _image;
    }

    public String getCode ( ) {
	return code;
    }

    public String getName ( ) {
	return name;
    }

    public int getType ( ) {
	return type;
    }

    public boolean isCdrom ( ) {
	return (code.length() >= 5  &&  code.substring(0,5).equals("cdrom"));
    }

    public boolean isAllsky ( ) {
	return all_sky;
    }

    public boolean isImage ( ) {
	return image;
    }

    public double getMagLimit ( ) {
	return mag_limit;
    }

    public double getFovLimit ( ) {
	return fov_limit;
    }

    // unify duplicated data
    // to be invoded just after loading
    public void unifyDuplication ( CatalogList list ) {
	if (code.equals("cdrom.usnoa10")  ||
	    code.equals("cdrom.usnoa10r")  ||
	    code.equals("cdrom.usnoa10b")  ||
	    code.equals("cdrom.usnoa20")  ||
	    code.equals("cdrom.usnoa20r")  ||
	    code.equals("cdrom.usnoa20b")  ||
	    code.equals("www.usnoa20")  ||
	    code.equals("www.usnoa20r")  ||
	    code.equals("www.usnoa20b")  ||
	    code.equals("www.usnoa10_vizier")  ||
	    code.equals("www.usnoa10r_vizier")  ||
	    code.equals("www.usnoa10b_vizier")) {
	    // USNO-A1.0, USNO-A2.0
	    list.unifyDuplication(5.0 / 3600.0, 1.5);
	} else if (code.equals("cdrom.gsc11")  ||
		   code.equals("www.gsc12")) {
	    // GSC 1.1, GSC 1.2
	    list.unifyDuplication();
	}
    }

    public CatalogList createCatalogList ( ) {
	if (code.equals("cdrom.gsc11")) {
	    return new CdromListGSC();
	} else if (code.equals("cdrom.usnoa10")) {
	    return new CdromListUSNOA10();
	} else if (code.equals("cdrom.usnoa10r")) {
	    CdromListUSNOA10 list = new CdromListUSNOA10();
	    list.setModeRmag();
	    return list;
	} else if (code.equals("cdrom.usnoa10b")) {
	    CdromListUSNOA10 list = new CdromListUSNOA10();
	    list.setModeBmag();
	    return list;
	} else if (code.equals("cdrom.usnoa20")) {
	    return new CdromListUSNOA20();
	} else if (code.equals("cdrom.usnoa20r")) {
	    CdromListUSNOA20 list = new CdromListUSNOA20();
	    list.setModeRmag();
	    return list;
	} else if (code.equals("cdrom.usnoa20b")) {
	    CdromListUSNOA20 list = new CdromListUSNOA20();
	    list.setModeBmag();
	    return list;
	} else if (code.equals("www.yale3")) {
	    return new CgiListYcbs();
	} else if (code.equals("www.tycho")) {
	    return new CgiListHIPTYC();
	} else if (code.equals("www.tycho_kyoto")) {
	    return new CgiListTycho();
	} else if (code.equals("www.gsc12")) {
	    return new CgiListGSC();
	} else if (code.equals("www.usnoa20")) {
	    return new CgiListUSNO();
	} else if (code.equals("www.usnoa20r")) {
	    CgiListUSNO list = new CgiListUSNO();
	    list.setModeRmag();
	    return list;
	} else if (code.equals("www.usnoa20b")) {
	    CgiListUSNO list = new CgiListUSNO();
	    list.setModeBmag();
	    return list;
	} else if (code.equals("www.usnoa20_vizier")) {
	    return new CgiListUSNOVizieR();
	} else if (code.equals("www.usnoa20r_vizier")) {
	    CgiListUSNOVizieR list = new CgiListUSNOVizieR();
	    list.setModeRmag();
	    return list;
	} else if (code.equals("www.usnoa20b_vizier")) {
	    CgiListUSNOVizieR list = new CgiListUSNOVizieR();
	    list.setModeBmag();
	    return list;
	}

	return null;
    }

    public ImageRGB loadImage ( Coor coor, double fov ) throws Exception {
	if (code.equals("www.dss1")) {
	    return new ImageCgiDSS(coor, fov);
	}

        return null;
    }
}
