/*
 * CdromListGSC.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;
import java.util.Vector;

public class CdromListGSC extends CdromList {
    private static final int dir_N = 24;
    private static final int dir_max[] = {
	 593, 1177, 1728, 2258, 2780, 3245, 3651, 4013, 
	4293, 4491, 4614, 4662, 5259, 5837, 6411, 6988, 
	7522, 8021, 8463, 8839, 9133, 9345, 9489, 9537
    };
    private static final String dir_name[] = {
	"n0000", "n0730", "n1500", "n2230", "n3000", "n3730", "n4500", "n5230", 
	"n6000", "n6730", "n7500", "n8230", "s0000", "s0730", "s1500", "s2230", 
	"s3000", "s3730", "s4500", "s5230", "s6000", "s6730", "s7500", "s8230"
    };

    public void load ( Coor center_coor, double fov, double mag, String[] path_list ) throws Exception {
	setUnit(fov);

	int i, j, k, p;
	String s;
	File file;

	restrictArea(center_coor, fov);

	String plate_date_id[] = new String[1518];
	String plate_date[] = new String[1518];

	// read the plate table
	byte b[] = new byte[115];
	file = checkCdrom(path_list, "TABLES/PLATES.TBL");
	DataInputStream fin = new DataInputStream(new BufferedInputStream(new FileInputStream(file)));
	fin.skip(28800);		// header
	for (i = 0 ; i < 1518 ; i++) {	// index
	    fin.readFully(b, 0, 115);
	    s = new String(b);
	    plate_date_id[i] = s.substring(9, 13);
	    plate_date[i] = s.substring(27, 36);
	}
	fin.close();

	Vector filename_list = new Vector();
	String filename;

	// read the region table
	b = new byte[48];
	file = checkCdrom(path_list, "TABLES/REGIONS.TBL");
	fin = new DataInputStream(new BufferedInputStream(new FileInputStream(file)));
	fin.skip(14400);		// header
	for (i = 0 ; i < 9537 ; i++) {	// index
	    p = 48;
	    try {
		fin.readFully(b, 0, 48);
	    } catch (EOFException e) {
		p = 0;
	    }
	    if (p == 48) {
		s = new String(b);
		int file_number = Integer.parseInt(s.substring(0,5).trim());

		Coor start_coor = new Coor(Integer.parseInt(s.substring(7,9).trim()), Integer.parseInt(s.substring(10,12).trim()), (double)(Float.valueOf(s.substring(13,18).trim()).floatValue()), (s.charAt(31) == '-'), Integer.parseInt(s.substring(32,34).trim()), 0, (double)(Float.valueOf(s.substring(35,39).trim()).floatValue()) * 60.0);
		Coor end_coor = new Coor(Integer.parseInt(s.substring(19,21).trim()), Integer.parseInt(s.substring(22,24).trim()), (double)(Float.valueOf(s.substring(25,30).trim()).floatValue()), (s.charAt(40) == '-'), Integer.parseInt(s.substring(41,43).trim()), 0, (double)(Float.valueOf(s.substring(44,48).trim()).floatValue()) * 60.0);

		if (overlapArea(start_coor, end_coor)) {
		    for (j = 0 ; j < dir_N ; j++) {
			if (file_number <= dir_max[j])
			    break;
		    }

		    filename = dir_name[j] + "/" + Astro.toStringZeroPadding(file_number, 4);
		    filename_list.addElement(filename);
		}
	    }
	}
	fin.close();

	CatalogData last_sd = new CatalogData();
	String last_id = "";

	// read star data in all selected files
	b = new byte[45];
	for (i = 0 ; i < filename_list.size() ; i++) {
	    filename = (String)filename_list.elementAt(i);

	    String disk = "volume 2 (southern part)";
	    if (filename.toLowerCase().charAt(0) == 'n'  ||  filename.substring(0,5).equalsIgnoreCase("s0000"))
		disk = "volume 1 (northern part)";
	    file = checkCdrom(path_list, "GSC/" + filename + ".GSC", disk);
	    fin = new DataInputStream(new BufferedInputStream(new FileInputStream(file)));
	    fin.skip(8640);		// header

	    // data
	    while (true) {
		p = 45;
		try {
		    fin.readFully(b, 0, 45);
		} catch (EOFException e) {
		    p = 0;
		}
		if (p < 45  ||  b[0] == ' ')
		    break;

		double m = b[29] - '0';
		if (b[28] == '1')
		    m += 10.0;
		m += (b[31] - '0') / 10.0;
		m += (b[32] - '0') / 100.0;
		if (b[28] == '-')
		    m = - m;

		if (m <= mag) {
		    s = new String(b);

		    CoorMapping coor = new CoorMapping((double)(Float.valueOf(s.substring(5,14).trim()).floatValue()), (double)(Float.valueOf(s.substring(14,23).trim()).floatValue()));
//		    double m = (double)(Float.valueOf(s.substring(28,33).trim()).floatValue());

		    if (inArea(coor)) {
			String id = filename.substring(filename.length() - 4, filename.length()) + "." + s.substring(1, 5);
			id = "GSC" + id.replace(' ', '0');

			String record = null;
			if (isModeSimple()) {
			    record = id + "  " + coor.toStringRA() + "  " + coor.toStringDecl()  + "  " + s.substring(28,33);
			} else if (isModeDetail()) {
			    record = id + "  R.A.=" + coor.toStringRA() + "  Decl.=" + coor.toStringDecl()  + " (+/-" + s.substring(23,28).trim() + "\")  Mag=" + s.substring(28,33) + " (+/-" + s.substring(33,37).trim() + ")  band=" + s.substring(37,39).trim() + "  class=" + s.substring(39,40).trim() + "  plate=" + s.substring(40,44) + "  flag=" + s.substring(44,45);

			    for (k = 0 ; k < 1518 ; k++) {
				if (plate_date_id[k].equals(s.substring(40,44))) {
				    record += "  Epoch=" + plate_date[k].substring(0, 7) + "19" + plate_date[k].substring(7, 9);
				    break;
				}
			    }
			}

			addStar(center_coor, coor, m, (isModeNull() ? null : id), record);

			// data duplication
			CatalogData sd = (CatalogData)elementLast();
			if (id.equals(last_id)) {
			    last_sd.append(sd);
			    removeLastElement();
			}
			last_id = id;
			last_sd = sd;
		    }
		}
	    }

	    fin.close();
	}
    }

    public CatalogData createCatalogData ( String s ) {
	if (s.length() < 3  ||  ! s.substring(0,3).equals("GSC"))
	    return null;

	CatalogData data = new CatalogData();
	data.setRecord(s);

	int p = s.indexOf(' ');
	data.setID(s.substring(0,p));
	s = s.substring(p).trim();

	if (s.length() >= 5  &&  s.substring(0,5).equals("R.A.=")) {
	    p = s.indexOf("R.A.=");
	    s = s.substring(p+5);
	    p = s.indexOf("Decl.=");
	    String ra_decl = s.substring(0,p-2);
	    s = s.substring(p+6);
	    p = s.indexOf('(');
	    ra_decl += " " + s.substring(0, p-1);
	    data.setCoor(Coor.createCoor(ra_decl));
	    p = s.indexOf("Mag=");
	    s = s.substring(p+4);
	    p = s.indexOf('(');
	    data.setMag(Astro.doubleValueOf(s.substring(0, p-1)));
	} else {
	    data.setCoor(Coor.createCoor(s));
	    p = s.indexOf("  ");
	    s = s.substring(p).trim();
	    p = s.indexOf("  ");
	    s = s.substring(p).trim();
	    data.setMag(Astro.doubleValueOf(s));
	}

	return data;
    }
}
