/*
 * CgiListUSNO.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class CgiListUSNO extends CgiList {
    protected static final String urlCGI = "http://asteroid.lowell.edu/cgi-bin/koehn/webnet";

    protected static final int USNO_V = 0;
    protected static final int USNO_R = 1;
    protected static final int USNO_B = 2;

    private int mode;

    public CgiListUSNO ( ) {
	super();

	mode = USNO_V;
    }

    public void setModeRmag ( ) {
	mode = USNO_R;
    }
    public void setModeBmag ( ) {
	mode = USNO_B;
    }

    public void load ( Coor center_coor, double fov, double mag ) throws Exception {
	try {
	    load(center_coor, fov, mag, mode);
	} catch (Exception e) {
	    throw e;
	}
    }

    public void loadRmag ( Coor center_coor, double fov, double mag ) throws Exception {
	try {
	    load(center_coor, fov, mag, USNO_R);
	} catch (Exception e) {
	    throw e;
	}
    }

    public void loadBmag ( Coor center_coor, double fov, double mag ) throws Exception {
	try {
	    load(center_coor, fov, mag, USNO_B);
	} catch (Exception e) {
	    throw e;
	}
    }

    private void load ( Coor center_coor, double fov, double mag, int mag_type ) throws Exception {
	setUnit(fov);

	String mag_str = "";
	if (mag > 30.0)
	    mag_str = "30.0";
	else if (-0.1 <= mag  &&  mag <= 0.1)
	    mag_str = "0.0";
	else {
	    mag_str = "" + mag;
	    if (mag_str.length() > 6)
		mag_str = mag_str.substring(0,6);
	}

	String arg = "rawidth=" + (int)(fov * 3600.0) + "&decwidth=" + (int)(fov * 3600.0) + "&ra=" + center_coor.getRA_h() + "+" + center_coor.getRA_m() + "+" + center_coor.getRA_s() + "&dec=" + (center_coor.getDecl() < 0.0 ? "-" : "") + center_coor.getAbsDecl_d() + "+" + center_coor.getAbsDecl_m() + "+" + center_coor.getAbsDecl_s() + "&redlo=-5.0&redhi=" + mag_str + "&bluelo=-5.0&bluehi=" + mag_str + "&cilo=-10.0&cihi=10.0&density=10000000.0&shape=Rectangular&sortby=Red+mag&sortdirection=Ascending&outstyle=ASCII&focallength=5.0&pixelsize=15.0&catalog=USNO-A1.0";

	try {
	    openConnectionPost(urlCGI, arg);
	} catch ( Exception e ) {
	    throw e;
	}

	String record = null;
	double mR;
	double mB;
	double mV;
	double m;

	try {
	    record = readLine();
	} catch ( Exception e ) {
	    throw e;
	}

	while (record != null) {
	    if (record.length() == 60) {
		if (record.charAt(0) == ' '  &&
		    '0' <= record.charAt(1)  &&  record.charAt(1) <= '2'  &&
		    '0' <= record.charAt(2)  &&  record.charAt(2) <= '9'  &&
		    record.charAt(3) == ' '  &&
		    '0' <= record.charAt(4)  &&  record.charAt(4) <= '9'  &&
		    '0' <= record.charAt(5)  &&  record.charAt(5) <= '9'  &&
		    record.charAt(6) == ' '  &&
		    '0' <= record.charAt(7)  &&  record.charAt(7) <= '9'  &&
		    '0' <= record.charAt(8)  &&  record.charAt(8) <= '9'  &&
		    record.charAt(9) == '.'  &&
		    '0' <= record.charAt(10)  &&  record.charAt(10) <= '9'  &&
		    '0' <= record.charAt(11)  &&  record.charAt(11) <= '9'  &&
		    '0' <= record.charAt(12)  &&  record.charAt(12) <= '9'  &&
		    record.charAt(13) == ' '  &&
		    record.charAt(14) == ' '  &&
		    (record.charAt(15) == '+'  ||  record.charAt(15) == '-')  &&
		    '0' <= record.charAt(16)  &&  record.charAt(16) <= '9'  &&
		    '0' <= record.charAt(17)  &&  record.charAt(17) <= '9'  &&
		    record.charAt(18) == ' '  &&
		    '0' <= record.charAt(19)  &&  record.charAt(19) <= '9'  &&
		    '0' <= record.charAt(20)  &&  record.charAt(20) <= '9'  &&
		    record.charAt(21) == ' '  &&
		    '0' <= record.charAt(22)  &&  record.charAt(22) <= '9'  &&
		    '0' <= record.charAt(23)  &&  record.charAt(23) <= '9'  &&
		    record.charAt(24) == '.'  &&
		    '0' <= record.charAt(25)  &&  record.charAt(25) <= '9'  &&
		    '0' <= record.charAt(26)  &&  record.charAt(26) <= '9'  &&
		    record.charAt(27) == ' '  &&
		    record.charAt(28) == ' '  &&
		    record.charAt(29) == ' '  &&
		    (record.charAt(30) == ' '  ||  ('0' <= record.charAt(30)  &&  record.charAt(30) <= '9'))  &&
		    '0' <= record.charAt(31)  &&  record.charAt(31) <= '9'  &&
		    record.charAt(32) == '.'  &&
		    '0' <= record.charAt(33)  &&  record.charAt(33) <= '9'  &&
		    (record.charAt(34) == ' '  ||  record.charAt(34) == 'e'  ||  record.charAt(34) == 'm')  &&
		    record.charAt(35) == ' '  &&
		    (record.charAt(36) == ' '  ||  ('0' <= record.charAt(36)  &&  record.charAt(36) <= '9'))  &&
		    '0' <= record.charAt(37)  &&  record.charAt(37) <= '9'  &&
		    record.charAt(38) == '.'  &&
		    '0' <= record.charAt(39)  &&  record.charAt(39) <= '9'  &&
		    (record.charAt(40) == ' '  ||  record.charAt(40) == 'e'  ||  record.charAt(40) == 'm')  &&
		    record.charAt(41) == ' '  &&
		    (record.charAt(42) == ' '  ||  ('0' <= record.charAt(42)  &&  record.charAt(42) <= '9'))  &&
		    (record.charAt(43) == ' '  ||  ('0' <= record.charAt(43)  &&  record.charAt(43) <= '9'))  &&
		    (record.charAt(44) == ' '  ||  ('0' <= record.charAt(44)  &&  record.charAt(44) <= '9'))  &&
		    (record.charAt(45) == ' '  ||  ('0' <= record.charAt(45)  &&  record.charAt(45) <= '9'))  &&
		    '0' <= record.charAt(46)  &&  record.charAt(46) <= '9'  &&
		    record.charAt(47) == '.'  &&
		    '0' <= record.charAt(48)  &&  record.charAt(48) <= '9'  &&
		    record.charAt(49) == ' '  &&
		    record.charAt(50) == ' '  &&
		    record.charAt(51) == ' '  &&
		    record.charAt(52) == ' '  &&
		    record.charAt(53) == ' '  &&
		    record.charAt(54) == ' '  &&
		    (record.charAt(55) == ' '  ||  ('0' <= record.charAt(55)  &&  record.charAt(55) <= '9'))  &&
		    (record.charAt(56) == ' '  ||  ('0' <= record.charAt(56)  &&  record.charAt(56) <= '9'))  &&
		    '0' <= record.charAt(57)  &&  record.charAt(57) <= '9'  &&
		    record.charAt(58) == '.'  &&
		    '0' <= record.charAt(59)  &&  record.charAt(59) <= '9') {

		    // R.A. and Decl.
		    CoorMapping coor = new CoorMapping(Integer.parseInt(record.substring(1,3)), Integer.parseInt(record.substring(4,6)), (double)(Float.valueOf(record.substring(7,13)).floatValue()), (record.charAt(15) == '-'), Integer.parseInt(record.substring(16,18)), Integer.parseInt(record.substring(19,21)), (double)(Float.valueOf(record.substring(22,27)).floatValue()));

		    // R mag
		    mR = ((double)(Float.valueOf(record.substring(30,34).trim()).floatValue()));
		    if (record.charAt(34) == 'm')
			mR = 99.9;
		    // B mag
		    mB = ((double)(Float.valueOf(record.substring(36,40).trim()).floatValue()));
		    if (record.charAt(40) == 'm')
			mB = 99.9;
		    // preliminary V mag
		    if (mR < 99.9) {
			if (mB < 99.9) {
			    mV = Astro.getUSNOVmag(mR, mB);
			} else {
			    mV = mR;
			}
		    } else {
			mV = mB;
		    }
		    // magnitude
		    if (mag_type == USNO_R) {
			if (mR < 99.9)
			    m = mR;
			else
			    m = mV;
		    } else if (mag_type == USNO_B) {
			if (mB < 99.9)
			    m = mB;
			else
			    m = mV;
		    } else {
			m = mV;
		    }

		    if (m <= mag) {
			if (isModeSimple()) {
			    record = record.substring(0,13) + "  " + record.substring(15,27) + "  " + record.substring(30,35) + "R  " + record.substring(36,41) + "B";
			    if (mV < 99.9  &&  mR < 99.9  &&  mB < 99.9) {
				record += "  (" + Astro.toString(mV, 5, 2) + "V)";
			    }
			} else if (isModeDetail()) {
			    record = "R.A.=" + record.substring(0,13) + "  Decl.=" + record.substring(15,27) + "  Mag(R)=" + record.substring(30,35) + "  Mag(B)=" + record.substring(36,41);
			    if (mV < 99.9  &&  mR < 99.9  &&  mB < 99.9) {
				record += "  (preliminary V mag=" + Astro.toString(mV, 5, 2) + ")";
			    }
			} else {
			    record = null;
			}

			addStar(center_coor, coor, m, null, record);
		    }
		}
	    }

	    try {
		record = readLine();
	    } catch ( Exception e ) {
		throw e;
	    }
	}
	close();
    }
}
