/*
 * Coor.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.util.*;

public class Coor {
    private double right_ascension;
    private double declination;
    private double distance;

    public Coor ( ) {
	this(0.0, 0.0, 1.0);
    }

    public Coor ( double ra, double decl ) {
	this(ra, decl, 1.0);
    }

    public Coor ( double ra, double decl, double dist ) {
	right_ascension = ra;
	declination = decl;
	distance = dist;
    }

    public Coor ( int ra_h, int ra_m, double ra_s, boolean south_flag, int decl_d, int decl_m, double decl_s ) {
	this(((double)ra_h + (double)ra_m / 60.0 + ra_s / 3600.0) * 15.0, (south_flag ? -1.0 : 1.0) * ((double)decl_d + (double)decl_m / 60.0 + decl_s / 3600.0), 1.0);
    }

    public Coor ( Coor coor ) {
	right_ascension = coor.right_ascension;
	declination = coor.declination;
	distance = coor.distance;
    }

    public Coor ( Xyz xyz ) {
	distance = Math.sqrt(xyz.getX() * xyz.getX() + xyz.getY() * xyz.getY() + xyz.getZ() * xyz.getZ());

	right_ascension = Math.atan2(xyz.getY(), xyz.getX()) / Astro.RAD;
	if (right_ascension < 0.0)
	    right_ascension += 360.0;
	declination = Math.asin(xyz.getZ() / distance) / Astro.RAD;
    }

    public static Coor createCoor ( String _s ) {
	String s = _s.trim();

	if (s.indexOf('h') >= 0  &&  s.indexOf('m') >= 0  &&  s.indexOf('s') >= 0) {
	    int ra_h = Integer.valueOf(s.substring(0,s.indexOf('h'))).intValue();
	    s = s.substring(s.indexOf('h')+1).trim();
	    int ra_m = Integer.valueOf(s.substring(0,s.indexOf('m'))).intValue();
	    s = s.substring(s.indexOf('m')+1).trim();
	    double ra_s = 0.0;
	    if (s.indexOf("s.") >= 0) {
		String ra_s_str = s.substring(0,s.indexOf('s')) + s.substring(s.indexOf('s')+1,s.indexOf(' '));
		ra_s = Astro.doubleValueOf(ra_s_str);
	    } else if (s.indexOf("s") >= 0) {
		ra_s = Astro.doubleValueOf(s.substring(0,s.indexOf('s')));
	    } else {
		ra_s = Astro.doubleValueOf(s.substring(0,s.indexOf(' ')));
	    }

	    s = s.substring(s.indexOf(' ')+1).trim();
	    boolean south_flag = (s.charAt(0) == '-');
	    if (s.charAt(0) == '+'  ||  s.charAt(0) == '-')
		s = s.substring(1);
	    int decl_d = 0;
	    if (s.indexOf("d") >= 0) {
		decl_d = Integer.valueOf(s.substring(0,s.indexOf('d'))).intValue();
		s = s.substring(s.indexOf('d')+1).trim();
	    } else {
		decl_d = Integer.valueOf(s.substring(0,s.indexOf('o'))).intValue();
		s = s.substring(s.indexOf('o')+1).trim();
	    }
	    int decl_m = Integer.valueOf(s.substring(0,s.indexOf('\''))).intValue();
	    s = s.substring(s.indexOf('\'')+1).trim();
	    double decl_s = 0.0;
	    if (s.indexOf("\".") >= 0) {
		String decl_s_str = s.substring(0,s.indexOf('"')) + s.substring(s.indexOf('\"')+1);
		decl_s = Astro.doubleValueOf(decl_s_str);
	    } else if (s.indexOf("\"") >= 0) {
		decl_s = Astro.doubleValueOf(s.substring(0,s.indexOf('"')));
	    } else {
		decl_s = Astro.doubleValueOf(s);
	    }

	    return new Coor(ra_h, ra_m, ra_s, south_flag, decl_d, decl_m, decl_s);
	}

	StringTokenizer st = new StringTokenizer(s);
	if (st.countTokens() == 2) {
	    String ra_str = st.nextToken();
	    String decl_str = st.nextToken();

	    int ra_h = Integer.valueOf(ra_str.substring(0,2)).intValue();
	    int ra_m = Integer.valueOf(ra_str.substring(2,4)).intValue();
	    double ra_s = Astro.doubleValueOf(ra_str.substring(4));
	    boolean south_flag = (decl_str.charAt(0) == '-');
	    int decl_d = Integer.valueOf(decl_str.substring(1,3)).intValue();
	    int decl_m = Integer.valueOf(decl_str.substring(3,5)).intValue();
	    double decl_s = Astro.doubleValueOf(decl_str.substring(5));

	    return new Coor(ra_h, ra_m, ra_s, south_flag, decl_d, decl_m, decl_s);
	}

	int ra_h = Integer.valueOf(st.nextToken()).intValue();
	int ra_m = Integer.valueOf(st.nextToken()).intValue();
	double ra_s = Astro.doubleValueOf(st.nextToken());
	s = st.nextToken();
	boolean south_flag = (s.charAt(0) == '-');
	if (s.equals("+")  ||  s.equals("-"))
	    s = st.nextToken();
	else if (s.charAt(0) == '+'  ||  s.charAt(0) == '-')
	    s = s.substring(1);
	int decl_d = Math.abs(Integer.valueOf(s).intValue());
	int decl_m;
	double decl_s;
	s = st.nextToken();
	if (s.indexOf('.') >= 0) {
	    double f = Math.abs(Astro.doubleValueOf(s));
	    decl_m = (int)f;
	    decl_s = (f - (double)decl_m) * 60.0;
	} else {
	    decl_m = Math.abs(Integer.valueOf(s).intValue());
	    decl_s = Math.abs(Astro.doubleValueOf(st.nextToken()));
	}

	return new Coor(ra_h, ra_m, ra_s, south_flag, decl_d, decl_m, decl_s);
    }

    public double getRA ( ) {
	return right_ascension;
    }

    public double getDecl ( ) {
	return declination;
    }

    public double getDistance ( ) {
	return distance;
    }

    public int getRA_h ( ) {
	return (int)Math.abs(right_ascension / 15.0 + 0.00000001);
    }

    public int getRA_m ( ) {
	return (int)Math.abs((right_ascension / 15.0 - getRA_h()) * 60.0 + 0.0000001);
    }

    public double getRA_s ( ) {
	return Math.abs((right_ascension / 15.0 - getRA_h()) * 60.0 - getRA_m()) * 60.0;
    }

    public int getAbsDecl_d ( ) {
	double d = declination;
	if (d < 0.0)
	    d = - d;
	return (int)Math.abs(d + 0.0000001);
    }

    public int getAbsDecl_m ( ) {
	double d = declination;
	if (d < 0.0)
	    d = - d;
	return (int)Math.abs((d - getAbsDecl_d()) * 60.0 + 0.000001);
    }

    public double getAbsDecl_s ( ) {
	double d = declination;
	if (d < 0.0)
	    d = - d;
	return Math.abs((d - getAbsDecl_d()) * 60.0 - getAbsDecl_m()) * 60.0;
    }

    public void setRA ( double ra ) {
	right_ascension = ra;
    }

    public void setDecl ( double decl ) {
	declination = decl;
    }

    // in degree
    public Coor residual ( Coor coor ) {
	double d_ra = Astro.norm(getRA() - coor.getRA(), 360.0);
	if (d_ra >= 180.0)
	    d_ra -= 360.0;
	d_ra *= Math.cos((getDecl() + coor.getDecl()) / 2.0 * Astro.RAD);
	double d_decl = getDecl() - coor.getDecl();
	return new Coor(d_ra, d_decl);
    }

    public double angular_distance ( Coor coor ) {
	if (right_ascension == coor.right_ascension  &&  declination == coor.declination)
	    return 0.0;

	return Math.acos(Math.sin(declination * Astro.RAD) * Math.sin(coor.declination * Astro.RAD) + Math.cos(declination * Astro.RAD) * Math.cos(coor.declination * Astro.RAD) * Math.cos((coor.right_ascension - right_ascension) * Astro.RAD) ) / Astro.RAD;
    }

    public double direction ( Coor coor ) {
	if (right_ascension == coor.right_ascension  &&  declination == coor.declination)
	    return 0.0;

	if (declination >= 90.0) {
	    double A = 180.0 - coor.right_ascension + right_ascension;
	    if (A < 0.0)
		A += 360.0;
	    if (A >= 0.0)
		A -= 360.0;
	    return A;
	}
	if (declination <= -90.0) {
	    double A = coor.right_ascension - right_ascension;
	    if (A < 0.0)
		A += 360.0;
	    if (A >= 0.0)
		A -= 360.0;
	    return A;
	}

	double ad = angular_distance(coor);
	double sinA = Math.cos(coor.declination * Astro.RAD) * Math.sin((coor.right_ascension - right_ascension) * Astro.RAD) / Math.sin(ad * Astro.RAD);
	double cosA = (Math.sin(coor.declination * Astro.RAD) - Math.sin(declination * Astro.RAD) * Math.cos(ad * Astro.RAD)) / (Math.cos(declination * Astro.RAD) * Math.sin(ad * Astro.RAD));
	double A = Math.atan2(sinA, cosA) / Astro.RAD;
	if (A < 0.0)
	    A += 360.0;
	return A;
    }

    // R.A. and Decl. -> x,y,z
    public Xyz convertXyz ( ) {
	double sin_a,cos_a,sin_d,cos_d;

	sin_a = Math.sin(right_ascension * Astro.RAD);
	cos_a = Math.cos(right_ascension * Astro.RAD);
	sin_d = Math.sin(declination * Astro.RAD);        
	cos_d = Math.cos(declination * Astro.RAD);    

	double x, y, z;
	x = distance * (cos_d * cos_a);
	y = distance * (cos_d * sin_a);
	z = distance * sin_d;
	Xyz xyz = new Xyz(x, y, z);

	return xyz;
    }

    // 3-D rotation: R.A. and Decl. -> x,y,z
    public Xyz rotate2xyz (double angle) {
	double sin_a,cos_a,sin_d,cos_d,sin_e,cos_e;

	sin_a = Math.sin(right_ascension * Astro.RAD);
	cos_a = Math.cos(right_ascension * Astro.RAD);
	sin_d = Math.sin(declination * Astro.RAD);        
	cos_d = Math.cos(declination * Astro.RAD);    
	sin_e = Math.sin(angle * Astro.RAD);
	cos_e = Math.cos(angle * Astro.RAD);

	double x, y, z;
	x = distance * (cos_d * cos_a);
	y = distance * (sin_d * sin_e + cos_d * sin_a * cos_e);
	z = distance * (sin_d * cos_e - cos_d * sin_a * sin_e);
	Xyz xyz = new Xyz(x, y, z);

	return xyz;
    }

    // 3-D rotation: R.A. and Decl. -> R.A. and Decl.
    public void rotate (double angle) {
	Xyz xyz = rotate2xyz(angle);
	Coor coor = new Coor(xyz);

	right_ascension = coor.right_ascension;
	declination = coor.declination;
    }

    // change precession
    public void precession (JulianDay jd_old, JulianDay jd_new) {
	Xyz xyz = convertXyz();
	xyz.precession(jd_old, jd_new);
	Coor coor = new Coor(xyz);
	right_ascension = coor.right_ascension;
	declination = coor.declination;
	distance = coor.distance;
    }

    public String toString ( ) {
	return toStringRA() + " " + toStringDecl() + " " + distance;
    }

    public String toStringRA ( ) {
	String RA = Astro.toStringZeroPadding(getRA_h(), 2) + " " + Astro.toStringZeroPadding(getRA_m(), 2) + " " + Astro.toStringZeroPadding(getRA_s(), 5, 2);

	return RA;
    }

    public String toStringDecl ( ) {
	String Decl = (getDecl() < 0.0 ? '-' : '+') + Astro.toStringZeroPadding(getAbsDecl_d(), 2) + " " + Astro.toStringZeroPadding(getAbsDecl_m(), 2) + " " + Astro.toStringZeroPadding(getAbsDecl_s(), 4, 2);

	return Decl;
    }
}
