/*
 * ImageBuffer.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.awt.Point;

class ImageBuffer {
    private float pixels[];
    private Size size;

    public ImageBuffer ( ) {
	pixels = null;
	size = new Size();
    }

    public ImageBuffer ( Size s ) throws Exception {
	try {
	    allocate(s);
	} catch (Exception e) {
	    throw e;
	}
    }

    public ImageBuffer ( ImageBuffer ib ) {
	pixels = ib.pixels;
	size = ib.size;
    }

    private void allocate ( Size s ) throws Exception {
	pixels = new float[s.getWidth() * s.getHeight()];
	if (pixels == null) {
	    System.err.println("Out of memory. Cannot allocate ImageBuffer.");
	    throw new Exception();
	}

	size = s;
    }

    public Size getSize ( ) {
	return size;
    }

    public int getWidth ( ) {
	return size.getWidth();
    }

    public int getHeight ( ) {
	return size.getHeight();
    }

    public void setValue ( Point pt, double val ) {
	if (pixels != null) {
	    if (0 <= pt.x  &&  pt.x < getWidth()  &&  0 <= pt.y  &&  pt.y < getHeight()) {
		pixels[pt.y * getWidth() + pt.x] = (float)val;
	    }
	}
    }

    public double getValue ( Point pt ) {
	if (pixels != null) {
	    if (0 <= pt.x  &&  pt.x < getWidth()  &&  0 <= pt.y  &&  pt.y < getHeight()) {
		return pixels[pt.y * getWidth() + pt.x];
	    }
	}
	return 0.0;
    }
}
