/*
 * ImageCgiDSS.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;
import java.net.*;

public class ImageCgiDSS extends ImageRGB {
    protected static final String urlCGI = "http://dss.mtk.nao.ac.jp/cgi-bin/dss/dss.csh";

    public ImageCgiDSS ( Coor coor, double fov ) throws Exception {
	super(load(coor, fov));
    }

    private static ImageFileGif load ( Coor coor, double fov ) throws Exception {
	fov *= 60.0;

	if (fov >= 60.01) {
	    System.err.println("Too large field of view to read DSS: " + String.valueOf(fov));
	    throw new Exception();
	}

	String data = "?equ=J2000&R.A.=" + coor.getRA_h() + "+" + coor.getRA_m() + "+" + coor.getRA_s() + "&Dec.=" + (coor.getDecl() < 0.0 ? "-" : "") + coor.getAbsDecl_d() + "+" + coor.getAbsDecl_m() + "+" + coor.getAbsDecl_s() + "&";
	if (fov > 59.99)
	    data += "widhei=60+60";
	else
	    data += "widhei=" + fov + "+" + fov;
	data += "&ver=dss1";

	URL url;
	try {
	    url = new URL(urlCGI + data);
	} catch ( IOException e ) {
	    System.err.println("Cannot access to: " + urlCGI);
	    throw e;
	}

	URLConnection uc = url.openConnection();

	String rec;
	String gif_file = "";
	int p = -1;
	InputStream is = uc.getInputStream();
	DataInputStream dis = new DataInputStream(is);
	while ((rec = dis.readLine()) != null  &&  p < 0) {
	    if (rec.length() > 43  &&  rec.substring(0,43).equals("<img src=\"http://dss.mtk.nao.ac.jp/dss/www/")) {
		rec = rec.substring(43);
		p = rec.indexOf('"');
		if (p >= 0)
		    gif_file = rec.substring(0,p);
	    }
	}
	dis.close();

	try {
	    url = new URL("http://dss.mtk.nao.ac.jp/dss/www/" + gif_file);
	} catch ( IOException e ) {
	    System.err.println("Cannot open file: " + "http://dss.mtk.nao.ac.jp/dss/www/" + gif_file);
	    throw e;
	}

	ImageFileGif gif_image = new ImageFileGif(url);

	return gif_image;
    }
}
