/*
 * MapFunction.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;

public class MapFunction {
    private double r;
    private double theta;
    private Position pos;

    public MapFunction ( ) {
	this(1.0, 0.0, new Position());
    }

    public MapFunction ( MapFunction mf ) {
	this(mf.r, mf.theta, mf.getPosition());
    }

    public MapFunction ( double R, double T, Position Pos ) {
	r = R;
	theta = T;
	pos = new Position(Pos);
    }

    public double getRate ( ) {
	return r;
    }

    public double getTheta ( ) {
	return theta;
    }

    public double getX ( ) {
	return pos.getX();
    }

    public double getY ( ) {
	return pos.getY();
    }

    public Position getPosition ( ) {
	return pos;
    }

    public void setRate ( double R ) {
	r = R;
    }

    public void setTheta ( double Theta ) {
	theta = Theta;
    }

    public void setPosition ( Position Pos ) {
	pos = Pos;
    }

    public Position map ( Position Pos ) {
	double st = Math.sin(theta / 180.0 * Math.PI);
	double ct = Math.cos(theta / 180.0 * Math.PI);

	double x = r * (Pos.getX() * ct - Pos.getY() * st) + pos.getX();
	double y = r * (Pos.getX() * st + Pos.getY() * ct) + pos.getY();

	return new Position(x, y);
    }

    public MapFunction inverse ( ) {
	MapFunction inv = new MapFunction();
	inv.r = 1.0 / r;
	inv.theta = 360.0 - theta;
	if (inv.theta == 360.0)
	    inv.theta = 0.0;
	double st = Math.sin(theta / 180.0 * Math.PI);
	double ct = Math.cos(theta / 180.0 * Math.PI);
	inv.pos = new Position(- (pos.getX() * ct + pos.getY() * st) / r, (pos.getX() * st - pos.getY() * ct) / r);
	return inv;
    }

    public String toString ( ) {
	return " (X, Y) = " + Astro.toString(getRate(), 7, 3).trim() + " * Rotate_[" + Astro.toString(getTheta(), 8, 3) + "] (x, y) + (" + Astro.toString(getX(), 7, 4) + ", " + Astro.toString(getY(), 7, 4) + ")";
    }
}
