/*
 * MapIndex.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.util.*;

public class MapIndex {
    protected Vector[][] map;

    private double Xmin;
    private double Xmax;
    private double Ymin;
    private double Ymax;
    private int Xcount;
    private int Ycount;
    private double Xmesh;
    private double Ymesh;

    public MapIndex ( double _Xmin, double _Xmax, double _Ymin, double _Ymax, int _Xcount, int _Ycount ) {
	map = new Vector[_Xcount][_Ycount];
	for (int i = 0 ; i < _Xcount ; i++) {
	    for (int j = 0 ; j < _Ycount ; j++) {
		map[i][j] = new Vector();
	    }
	}

	Xmin = _Xmin;
	Xmax = _Xmax;
	Ymin = _Ymin;
	Ymax = _Ymax;
	Xcount = _Xcount;
	Ycount = _Ycount;
	Xmesh = (Xmax - Xmin) / (double)Xcount;
	Ymesh = (Ymax - Ymin) / (double)Ycount;
    }

    public int getFirstIndex ( Position pos ) {
	int x = (int)((pos.getX() - Xmin) / Xmesh);
	if (x < 0)
	    x = 0;
	if (x >= Xcount)
	    x = Xcount - 1;
	return x;
    }

    public int getSecondIndex ( Position pos ) {
	int y = (int)((pos.getY() - Ymin) / Ymesh);
	if (y < 0)
	    y = 0;
	if (y >= Ycount)
	    y = Ycount - 1;
	return y;
    }

    public void addElement ( MapElement elm ) {
	int i = getFirstIndex(elm.getPosition());
	int j = getSecondIndex(elm.getPosition());
	map[i][j].addElement(elm);
    }

    public Vector getAllElements ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < Xcount ; i++) {
	    for (int j = 0 ; j < Ycount ; j++) {
		for (int k = 0 ; k < map[i][j].size() ; k++) {
		    list.addElement(map[i][j].elementAt(k));
		}
	    }
	}
	return list;
    }

    public MapElement findElementByName ( String name, Position pos ) {
	int i = getFirstIndex(pos);
	int j = getSecondIndex(pos);

	for (int k = 0 ; k < map[i][j].size() ; k++) {
	    MapElement elm = (MapElement)map[i][j].elementAt(k);
	    if (elm.getName().equals(name))
		return elm;
	}

	return null;
    }

    public void removeElementByName ( String name, Position pos ) {
	int i = getFirstIndex(pos);
	int j = getSecondIndex(pos);

	for (int k = 0 ; k < map[i][j].size() ; k++) {
	    MapElement elm = (MapElement)map[i][j].elementAt(k);
	    if (elm.getName().equals(name)) {
		map[i][j].removeElementAt(k);
		return;
	    }
	}
    }

    public Vector findElementByPosition ( Position pos, double dx, double dy ) {
	Position pos_min = new Position(pos.getX() - dx, pos.getY() - dy);
	Position pos_max = new Position(pos.getX() + dx, pos.getY() + dy);
	int i_min = getFirstIndex(pos_min);
	int j_min = getSecondIndex(pos_min);
	int i_max = getFirstIndex(pos_max);
	int j_max = getSecondIndex(pos_max);

	Vector list = new Vector();

	for (int i = i_min ; i <= i_max ; i++) {
	    for (int j = j_min ; j <= j_max ; j++) {
		for (int k = 0 ; k < map[i][j].size() ; k++)
		    list.addElement(map[i][j].elementAt(k));
	    }
	}

	return list;
    }
}
