/*
 * PrintManager.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;

public class PrintManager {
    private PrintStream ps;
    private boolean output_to_console;
    private boolean output_to_err;

    public PrintManager ( ) {
	ps = null;
	output_to_console = true;
	output_to_err = false;
    }

    public void close ( ) {
	if (ps != null)
	    ps.close();
	ps = null;
    }

    public void openFile( String filename ) throws Exception {
	close();

	if (filename != null  &&  filename.length() > 0) {
	    FileOutputStream fos = new FileOutputStream(filename);
	    ps = new PrintStream(fos);
	}
    }

    public void outputToConsole ( boolean f ) {
	output_to_console = f;
    }

    public void outputToErr ( boolean f ) {
	output_to_err = f;
    }

    public PrintStream getPrintStreamOfFile ( ) {
	return ps;
    }

    public void print ( String s ) {
	if (output_to_console)
	    System.out.print(s);
	if (output_to_err)
	    System.err.print(s);
	if (ps != null)
	    ps.print(s);
    }

    public void println ( String s ) {
	if (output_to_console)
	    System.out.println(s);
	if (output_to_err)
	    System.err.println(s);
	if (ps != null)
	    ps.println(s);
    }

    public void printObject ( Printable obj ) {
	if (output_to_console)
	    obj.print(System.out);
	if (output_to_err)
	    obj.print(System.err);
	if (ps != null)
	    obj.print(ps);
    }
}
