/*
 * Quadrics.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class Quadrics {
    // z = A + Bx + Cy + Dx^2 + Ey^2 + Fxy
    private double A;
    private double B;
    private double C;
    private double D;
    private double E;
    private double F;

    private SimultaneousEquation se;
    private int data_count;

    public Quadrics ( ) {
	A = B = C = D = E = F = 0.0;

	se = new SimultaneousEquation(6);
	data_count = 0;
    }

    public double getValue ( int x, int y ) {
	return getValue((double)x, (double)y);
    }

    public double getValue ( double x, double y ) {
	return A + B * x + C * y + D * x * x + E * y * y + F * x * y;
    }

    // set one data for the least square method
    public void setData ( double x, double y, double z ) {
	data_count++;

	double x2 = x * x;
	double x3 = x2 * x;
	double y2 = y * y;
	double y3 = y2 * y;

	se.coef[0][1] += x;
	se.coef[0][2] += y;
	se.coef[0][3] += x2;
	se.coef[0][4] += y2;
	se.coef[0][5] += x * y;
	se.coef[0][6] += - z;
	se.coef[1][3] += x3;
	se.coef[1][4] += x * y2;
	se.coef[1][5] += x2 * y;
	se.coef[1][6] += - x * z;
	se.coef[2][4] += y3;
	se.coef[2][6] += - y * z;
	se.coef[3][3] += x2 * x2;
	se.coef[3][4] += x2 * y2;
	se.coef[3][5] += x3 * y;
	se.coef[3][6] += - x2 * z;
	se.coef[4][4] += y2 * y2;
	se.coef[4][5] += x * y3;
	se.coef[4][6] += - y2 * z;
	se.coef[5][6] += - x * y * z;
    }

    // calculate 6 coefficients in the least square method
    public void solve ( ) {
	if (data_count > 0) {
	    se.coef[0][0] = (double)data_count;
	    se.coef[1][0] = se.coef[0][1];
	    se.coef[1][1] = se.coef[0][3];
	    se.coef[1][2] = se.coef[0][5];
	    se.coef[2][0] = se.coef[0][2];
	    se.coef[2][1] = se.coef[0][5];
	    se.coef[2][2] = se.coef[0][4];
	    se.coef[2][3] = se.coef[1][5];
	    se.coef[2][5] = se.coef[1][4];
	    se.coef[3][0] = se.coef[0][3];
	    se.coef[3][1] = se.coef[1][3];
	    se.coef[3][2] = se.coef[1][5];
	    se.coef[4][0] = se.coef[0][4];
	    se.coef[4][1] = se.coef[1][4];
	    se.coef[4][2] = se.coef[2][4];
	    se.coef[4][3] = se.coef[3][4];
	    se.coef[5][0] = se.coef[0][5];
	    se.coef[5][1] = se.coef[1][5];
	    se.coef[5][2] = se.coef[1][4];
	    se.coef[5][3] = se.coef[3][5];
	    se.coef[5][4] = se.coef[4][5];
	    se.coef[5][5] = se.coef[3][4];

	    se.solve();

	    A = se.getAnswer(0);
	    B = se.getAnswer(1);
	    C = se.getAnswer(2);
	    D = se.getAnswer(3);
	    E = se.getAnswer(4);
	    F = se.getAnswer(5);
	}
    }

    public String toString ( ) {
	return toString("z");
    }

    public String toString ( String z ) {
	return z + " = " + A + " + " + B + " x + " + C + " y + " + D + " x^2 + " + E + " y^2 + " + F + " xy";
    }
}
