/*
 * SimultaneousEquation.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class SimultaneousEquation {
    public double coef[][];	// coefficient matrix (right edge is constant term)
    private int dimension;

    public SimultaneousEquation ( int dim ) {
	dimension = dim;
	coef = new double[dim][dim + 1];

	for (int i = 0 ; i < dim ; i++) {
	    for (int j = 0 ; j < dim + 1 ; j++)
		coef[i][j] = 0.0;
	}
    }

    public double getAnswer ( int i ) {
	return coef[i][i+1];
    }

    // solve simulatenous equation
    // the answer of line i of matrix will be in coef[i][i+1]
    // * it should consider the case coefficient is 0.0
    public void solve () {
	solve(dimension);
    }

    private void solve (int k) {
	int i, j;

	if (k == 1) {
	    coef[0][1] = - coef[0][1] / coef[0][0];
	    return;
	}

	for (i = 0 ; i < k ; i++) {
	    for (j = 0 ; j < k + 1 ; j++) {
		if (j != k - 1)
		    coef[i][j] /= coef[i][k - 1];
	    }
	}
	for (i = 0 ; i < k - 1 ; i++) {
	    for (j = 0 ; j < k - 1 ; j++)
		coef[i][j] -= coef[k - 1][j];
	    coef[i][k - 1] = coef[i][k] - coef[k - 1][k];
	}

	solve(k - 1);

	for (j = 0 ; j < k - 1 ; j++) {
	    coef[k - 1][j] *= coef[j][j + 1];
	    coef[k - 1][k] += coef[k - 1][j];
	}
	coef[k - 1][k] = - coef[k - 1][k];
    }
}
