/*
 * StarListIndex.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.awt.Point;
import java.util.Vector;

public class StarListIndex {
    private static final double overlap = 0.25;

    protected Size mesh_size;
    protected double mesh_unit;
    protected Position positionA;
    protected Position positionB;
    protected StarList table[][];

    protected Vector pt_list;
    protected int pt_index;

    // dummy constructor
    protected StarListIndex ( ) {
    }

    public StarListIndex ( StarList list, Position pos1, Position pos2, double unit, MapFunction mf ) {
	mesh_unit = unit;
	positionA = new Position(pos1);
	positionB = new Position(pos2);
	mesh_size = new Size((int)((pos2.getX() - pos1.getX()) / mesh_unit) + 1, (int)((pos2.getY() - pos1.getY()) / mesh_unit) + 1);
	table = new StarList[mesh_size.getWidth()][mesh_size.getHeight()];

	pt_list = new Vector();
	pt_index = 0;

	for (int i = 0 ; i < mesh_size.getWidth() ; i++) {
	    for (int j = 0 ; j < mesh_size.getHeight() ; j++) {
		table[i][j] = new StarList();
	    }
	}

	for (int i = 0 ; i < list.size() ; i++) {
	    StarData sd = list.elementAt(i);
	    Point pt = findIndex(sd, mf);
	    table[pt.x][pt.y].addElement(sd);
	}
    }

    public Point findIndex ( StarData sd, MapFunction mf ) {
	return findIndex(sd.getPosition(), mf);
    }

    public Point findIndex ( Position p, MapFunction mf ) {
	Position pos = mf.map(p);
	Point pt = new Point((int)((pos.getX() - positionA.getX()) / mesh_unit), (int)((pos.getY() - positionA.getY()) / mesh_unit));
	if (pt.x < 0)
	    pt.x = 0;
	if (mesh_size.getWidth() <= pt.x)
	    pt.x = mesh_size.getWidth() - 1;
	if (pt.y < 0)
	    pt.y = 0;
	if (mesh_size.getHeight() <= pt.y)
	    pt.y = mesh_size.getHeight() - 1;
	return pt;
    }

    public StarList findList ( StarData sd, MapFunction mf ) {
	Point pt = findIndex(sd, mf);
	return table[pt.x][pt.y];
    }

    public StarList getFirstList ( StarData sd, MapFunction mf ) {
	return getFirstList(sd.getPosition(), mf);
    }

    public StarList getFirstList ( Position p, MapFunction mf ) {
	Point pt0 = findIndex(p, mf);
	Point dpt = new Point(0, 0);

	pt_list = new Vector();
	Position pos = mf.map(p);

	for (dpt.y = -1 ; dpt.y <= 1 ; dpt.y++) {
	    for (dpt.x = -1 ; dpt.x <= 1 ; dpt.x++) {
		Point pt = new Point(pt0.x + dpt.x, pt0.y + dpt.y);
		if (0 <= pt.x  &&  pt.x < mesh_size.getWidth()  &&  0 <= pt.y  &&  pt.y < mesh_size.getHeight()) {
		    if ((double)pt.x * mesh_unit - mesh_unit * overlap + positionA.getX() <= pos.getX()  &&  pos.getX() < (double)(pt.x + 1) * mesh_unit + mesh_unit * overlap + positionA.getX()  &&  (double)pt.y * mesh_unit - mesh_unit * overlap + positionA.getY() <= pos.getY()  &&  pos.getY() < (double)(pt.y + 1) * mesh_unit + mesh_unit * overlap + positionA.getY())
			pt_list.addElement(pt);
		}
	    }
	}

	pt_index = 0;

	return getNextList();
    }

    public StarList getNextList ( ) {
	if (pt_index < pt_list.size()) {
	    Point pt = (Point)pt_list.elementAt(pt_index);
	    pt_index++;

	    return table[pt.x][pt.y];
	}

	return null;
    }

    public StarList getAllList ( ) {
	StarList list = new StarList();
	for (int i = 0 ; i < mesh_size.getWidth() ; i++) {
	    for (int j = 0 ; j < mesh_size.getHeight() ; j++) {
		list.add(table[i][j]);
	    }
	}
	return list;
    }

    public void removeOutOfArea ( MapFunction mf ) {
	Point pt = new Point(0,0);
	for (pt.y = 0 ; pt.y < mesh_size.getHeight() ; pt.y++)
	    removeOutOfArea(mf, pt);
	pt.x = mesh_size.getWidth() - 1;
	for (pt.y = 0 ; pt.y < mesh_size.getHeight() ; pt.y++)
	    removeOutOfArea(mf, pt);
	pt.y = 0;
	for (pt.x = 0 ; pt.x < mesh_size.getWidth() ; pt.x++)
	    removeOutOfArea(mf, pt);
	pt.y = mesh_size.getHeight() - 1;
	for (pt.x = 0 ; pt.x < mesh_size.getWidth() ; pt.x++)
	    removeOutOfArea(mf, pt);
    }

    private void removeOutOfArea ( MapFunction mf, Point pt ) {
	for (int i = 0 ; i < table[pt.x][pt.y].size() ; i++) {
	    StarData sd = table[pt.x][pt.y].elementAt(i);
	    Point sd_pt = findIndex(sd, mf);

	    if (sd_pt.x < 0  ||  mesh_size.getWidth() <= sd_pt.x  ||  sd_pt.y < 0  ||  mesh_size.getHeight() <= sd_pt.y) {
		table[pt.x][pt.y].removeElementAt(i);
		i--;
	    }
	}
    }

    // in case of data duplication (one star actually but several stars recorded)
    public void unify ( MapFunction mf, double dr ) {
	blend(mf, dr, 999.9, false);
    }
    public void unify ( MapFunction mf, double dr, double dm ) {
	blend(mf, dr, dm, false);
    }

    // in case of stars blending (several stars actually but one star apparently)
    public void blend ( MapFunction mf, double dr ) {
	blend(mf, dr, 999.9, true);
    }
    public void blend ( MapFunction mf, double dr, double dm ) {
	blend(mf, dr, dm, true);
    }

    private void blend ( MapFunction mf, double dr, double dm, boolean calculate_blended_magnitude_flag ) {
	for (int i = 0 ; i < mesh_size.getWidth() ; i++) {
	    for (int j = 0 ; j < mesh_size.getHeight() ; j++) {
		for (int k = 0 ; k < table[i][j].size() ; k++) {
		    StarData sd0 = table[i][j].elementAt(k);
		    Position pos0 = mf.map(sd0.getPosition());

		    double m = Math.pow(Astro.MAG_STEP, - sd0.getMag());

		    for (int l = k + 1 ; l < table[i][j].size() ; l++) {
			StarData sd = table[i][j].elementAt(l);
			Position pos = mf.map(sd.getPosition());

			if ((pos.getX() - pos0.getX()) * (pos.getX() - pos0.getX()) + (pos.getY() - pos0.getY()) * (pos.getY() - pos0.getY()) < dr * dr  &&  Math.abs(sd0.getMag() - sd.getMag()) < dm) {
			    sd0.append(sd);
			    table[i][j].removeElementAt(l);
			    l--;

			    m += Math.pow(Astro.MAG_STEP, - sd.getMag());
			}
		    }

		    if (calculate_blended_magnitude_flag)
			sd0.setMag(- Math.log(m) / Math.log(Astro.MAG_STEP));
		}
	    }
	}
    }

    // find a counterpart within the nearby area
    // the original starlist must be sorted
    // the found data will be removed
    public StarData extractPair ( StarData data, MapFunction mf, Position df_pos, double d_dist, double d_mag ) {
	return extractPair(data, mf, df_pos, d_dist, d_mag, false);
    }

    public StarData extractPairBrightest ( StarData data, MapFunction mf, Position df_pos, double d_dist, double d_mag ) {
	return extractPair(data, mf, df_pos, d_dist, d_mag, true);
    }

public StarData extractPair ( StarData data, MapFunction mf, Position df_pos, double d_dist, double d_mag, boolean pair_with_brightest ) {
	StarData pair_data = null;

	StarList pair_list = null;
	int pair_index = -1;
	double pair_mag = 99.9;
	double pair_score = 0.0;

	StarList list = getFirstList(data, new MapFunction());
	while (list != null) {
	    for (int i = 0 ; i < list.size() ; i++) {
		StarData sd = list.elementAt(i);
		if (sd.getMag() > data.getMag() + d_mag)
		    break;
		if (sd.getMag() > data.getMag() - d_mag) {
		    Position pos = mf.map(sd.getPosition());
		    double d = Math.sqrt((pos.getX() - (data.getX() + df_pos.getX())) * (pos.getX() - (data.getX() + df_pos.getX())) + (pos.getY() - (data.getY() + df_pos.getY())) * (pos.getY() - (data.getY() + df_pos.getY())));
		    if (d < d_dist) {
			if (pair_with_brightest) {
			    if (pair_list == null  ||  sd.getMag() < pair_mag) {
				pair_list = list;
				pair_index = i;
				pair_mag = sd.getMag();
			    }
			} else {
			    double dm = sd.getMag() - data.getMag();
			    double score = Math.exp(- d * d / 2.0 / d_dist / d_dist) * Math.exp(- dm * dm / 2.0 / d_mag / d_mag);
			    if (pair_list == null  ||  score > pair_score) {
				pair_list = list;
				pair_index = i;
				pair_score = score;
			    }
			}
		    }
		}
	    }

	    list = getNextList();
	}

	if (pair_list != null) {
	    pair_data = pair_list.elementAt(pair_index);
	    pair_list.removeElementAt(pair_index);
	}

	return pair_data;
    }
}
